package nl.tue.id.tim;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.swing.*;

import com.valhalla.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.*;
import com.valhalla.jbother.plugins.events.*;
import com.valhalla.pluginmanager.*;

/**
 * <p>TIM File Transfer plugin for JBother.</p>
 *
 * <p>Description: This plugin uses a simple http server for transfering files.
 * it monitors incoming and outgoing messages and reacts on those that started
 * with "/file". A "/file" message is expected to come with a url that links to
 * a file. The sender first upload selected file to the root of the http server
 * (a cache under the users profile directory), then form a url to this file,
 * send a "/file [url]" to the buddy in conversation. The receiver get the url
 * from the message, download the file and send a confirmation message back to
 * the sender.
 * <br> This plugin also addes a file transfer button to chat panels.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jun Hu</p>
 *
 * <p>Company: ID, TU/e</p>
 *
 * @author Jun Hu
 * @version 1.0
 */
public class FileTransfer
    implements Plugin, PluginEventListener {

  /**
   * i18n translation resources.
   */
  private ResourceBundle resources = JBother.resources;

  /**
   * a simple http server for tranfering files.
   */
  HttpServer server = new HttpServer();

  /**
   * root directory of the simple http server.
   */
  String root = "";

  public FileTransfer() {
  }

  /**
   * Initiate the plugin.
   * <br> add this plugin to the PluginChain, start the httpServer, add a
   * FileTransferButton to the ChatPanel.
   *
   * @return boolean. if the plugin is sucessfully loaded, retun true.
   */
  public boolean init() {
    PluginChain.addListener(this);

    String p = System.getProperty("port");
    if (p != null) {
      try {
        int port = Integer.parseInt(p);
        server.setPort(port);
      }
      catch (NumberFormatException ex1) {
      }
    }
    File cacheDir = new File(JBother.profileDir, "cache");
    cacheDir.mkdirs();
    cacheDir.deleteOnExit();

    root = cacheDir.getPath();
    server.setRoot(root);

    try {

      server.start();
    }
    catch (IOException ex) {
      Logger.debug("* ERROR * Web server could not start on port:" +
                   server.getPort());
    }

    FileTransferButton.reloadButtons();
    return true;
  }

  /**
   * Unloads the plugin. remove this plugin from the PluginChain, stop the web
   * server, remove file transfer buttons from ChatPanels.
   */
  public void unload() {
    PluginChain.removeListener(this);
    try {
      server.stop();
    }
    catch (IOException ex) {
    }
    FileTransferButton.removeButtons();
  }

  /**
   * This is the event handling interface defined in JBother Plugin.
   * File Transfer monitors the conversation events distributed from ChatPanel.
   *
   * @param pluginEvent PluginEvent
   */
  public void handleEvent(PluginEvent pluginEvent) {
    if (pluginEvent instanceof ConversationEvent) {
      conversationHandler( (ConversationEvent) pluginEvent);
    }
  }

  /**
   * conversationHandler processes the conversation events from ChatPanels. If
   * RECEIVEING_MESSAGE, check if the message is a file trasfer notification
   * that starts with "/file".
   * <br> If SENDING_MESSAGE and the message is "/file", then start a dialogue
   * to ask the user for the file to send, upload the selected file to the
   * httpServer root (<profile>/cache), form an http url link to this file,
   * modify the message to "/file [url]>".
   * <br> if the coversation is just initialized, add a file transfer button to
   * the ChatPanel.
   *
   * @param pluginEvent ConversationEvent
   */
  private void conversationHandler(ConversationEvent pluginEvent) {
    ChatPanel conversation = null;
    if (pluginEvent.getSource() instanceof ChatPanel) {
      conversation = (ChatPanel) pluginEvent.getSource();
    }
    if (conversation == null) {
      return;
    }
    BuddyStatus buddy = conversation.getBuddy();
    String content = pluginEvent.getContent();
    if (pluginEvent.getType() == ConversationEvent.Type.RECEIVEING_MESSAGE) {
      if (content.startsWith("/file ")) {
        final String f = content.replaceFirst("^\\/file ", "");
        final ChatPanel chat = conversation;
        (new Thread() {
          public void run() {
            try {
              downloadFile(f, chat);
            }
            catch (IOException ex) {
              Logger.debug(ex.getMessage());
            }

          }
        }).start();

      }
    }
    else if (pluginEvent.getType() == ConversationEvent.Type.SENDING_MESSAGE) {
      if (content.startsWith("/file") && content.trim().length() == 5) {
        String url = uploadFile();
        if (url != null) {
          pluginEvent.setContent("/file " + url);
        }
      }
    }
    else if (pluginEvent.getType() == ConversationEvent.Type.INIT) {
      new FileTransferButton(conversation, this);
      ;
    }
  }

  /**
   * uploadFile the file will be uploaded to a cahce directory that serves as
   * http root. the cache directory is named "cache" under the current user's
   * profile All the uploaded files will be removed when the virtual machine
   * terminates.
   *
   * @return String, formatted url link to the file uploaded.
   */
  public String uploadFile() {
    final JFileChooser fileChooser = new JFileChooser();
    fileChooser.setDialogType(fileChooser.SAVE_DIALOG);
    fileChooser.setDialogTitle("Which file to send? ");
    fileChooser.setMultiSelectionEnabled(false);
    String url = null;
    if (JFileChooser.APPROVE_OPTION ==
        fileChooser.showDialog(null, "Send")) {

      File selected = fileChooser.getSelectedFile();
      String filename = selected.getName();

      try {
        File copyto = new File(root + "/" + filename);
        copyto.deleteOnExit();
        copy(selected, copyto);
        try {
          InetAddress localhost = InetAddress.getLocalHost();
          String ip = localhost.getHostAddress();

          url = "http://" + ip + ":" + server.getPort() + "/" +
              URLEncoder.encode(filename, "UTF-8");

        }
        catch (Exception ex) {
        }
      }
      catch (IOException ex) {
      }
    }
    return url;
  }

  /**
   * download the file, when finish send a confirmation message to the butty in
   * chat.
   *
   * @param f String, the url link to the file to download. If the url is
   *   malformed, nothing will happen.
   * @param chat ChatPanel, to which the confirmation message will be sent.
   * @throws IOException IOException, if anything wrong during downloading, an
   *   IOException will be thrown.
   */
  public void downloadFile(String f, ChatPanel chat) throws IOException {
    URL url = null;
    try {
      url = new URL(f);
    }
    catch (MalformedURLException ex) {
      Logger.logException(ex);
      return;
    }
    InputStream uin = null;
    int size = -1;
    try {
      size = url.openConnection().getContentLength();
      uin = url.openStream();
    }
    catch (IOException ex1) {
      Logger.logException(ex1);
      return;
    }

    final JFileChooser fileChooser = new JFileChooser();
    String filename = URLDecoder.decode(url.getFile(),
                                        "UTF-8");
    fileChooser.setSelectedFile(new File(filename));
    fileChooser.setDialogType(fileChooser.SAVE_DIALOG);
    fileChooser.setDialogTitle("Accept " + filename + " ?");
    fileChooser.setMultiSelectionEnabled(false);
    if (JFileChooser.APPROVE_OPTION ==
        fileChooser.showDialog(null, "Download")) {

      File selected = fileChooser.getSelectedFile();
      if (selected.exists()) {
        if (JOptionPane.showConfirmDialog(
            null,
            "File \"" + selected.getName() + "\" already exists. Overwrite?",
            "Destination Download File",
            JOptionPane.YES_NO_OPTION,
            JOptionPane.QUESTION_MESSAGE) == JOptionPane.NO_OPTION) {
          return;
        }
      }

      InputStream inn = uin;

      if (size > 0) {
        inn = new ProgressMonitorInputStream(null,
                                             "Receiving file : " +
                                             selected.getName() +
                                             "...",
                                             uin);
        ( (ProgressMonitorInputStream) inn).getProgressMonitor().
            setMillisToDecideToPopup(1000);
        ( (ProgressMonitorInputStream) inn).getProgressMonitor().setMaximum(
            size);
      }

      BufferedInputStream in = new BufferedInputStream(inn);

      FileOutputStream file = null;
      file = new FileOutputStream(selected.getPath());
      byte[] buffer = new byte[1024];
      int len = 0;

      do {
        len = in.read(buffer);
        //len = in.read();
        if (len != -1) {
          file.write(buffer, 0, len);
          //file.write(len);
        }
      }
      while (len != -1);

      file.flush();
      in.close();
      file.close();

      Logger.debug("Downloaded " + filename + " !");
      chat.directSend("Received file: " + filename, true);
    }
  }

  /**
   * copy one file to another. no worning if the destination file exists. use it
   * with care.
   *
   * @param src File
   * @param dst File
   * @throws IOException
   */
  private void copy(File src, File dst) throws IOException {
    InputStream fin = new FileInputStream(src);
    OutputStream out = new FileOutputStream(dst);
    ProgressMonitorInputStream in = new ProgressMonitorInputStream(null,
        "Copying file to cache ...", fin);
    in.getProgressMonitor().setMillisToDecideToPopup(1000);

    // Transfer bytes from in to out
    byte[] buf = new byte[1024];
    int len;
    while ( (len = in.read(buf)) > 0) {
      out.write(buf, 0, len);
    }
    in.close();
    out.close();
  }

}


