package nl.tue.id.tim;

import java.io.*;
import java.net.*;
import java.util.regex.*;

import javax.swing.*;

import com.valhalla.*;

/**
 * <p>A simple http server that implements HTTP/1.0 GET. </p>
 *
 * <p>Description: The http server has its default port at 8080, and the default
 * root directory at where the program is started. All the url links are
 * supposed to be UTF-8 encoded. </p>
 *
 * <p>Copyright: Copyright (c) 2005 Jun Hu</p>
 *
 * <p>Company: ID, TU/e</p>
 *
 * @author Jun Hu
 * @version 1.0
 */
public class HttpServer
    implements Runnable {
  public HttpServer() {
    try {
      jbInit();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  /**
   * port number of the http server.
   */
  private int port = 8080;
  /**
   * the root directory of the http server.
   */
  private String root = "./";

  /**
   * server socket
   */
  ServerSocket ss = null;
  /**
   * a switch for start and stop the http server.
   */
  boolean running = false;

  /**
   * for testing as a standalone server.
   *
   * @param argv String[]
   * @throws IOException
   */
  public static void main(String argv[]) throws IOException {
    (new HttpServer()).start();
  }

  /**
   * start the http server as a new thread.
   *
   * @throws IOException
   */
  public void start() throws IOException {
    if (ss == null) {
      ss = new ServerSocket(port);
      Logger.debug("Http server started at port " + port);
    }
    running = true;
    (new Thread(this)).start();

  }

  /**
   * stop the http server.
   *
   * @throws IOException
   */
  public void stop() throws IOException {
    if (ss != null) {
      ss.close();
      ss = null;
      running = false;
      Logger.debug("Http server stopped at port " + port);
    }
  }

  /**
   * setter for the port.
   *
   * @param port int
   */
  public void setPort(int port) {
    this.port = port;
  }

  /**
   * setter for the root.
   *
   * @param root String
   */
  public void setRoot(String root) {
    if (!root.endsWith("/")) {
      root = root + "/";
    }
    this.root = root;
  }

  /**
   * return the root number.
   *
   * @return int
   */
  public int getPort() {
    return port;
  }

  /**
   * return the root directory.
   *
   * @return String
   */
  public String getRoot() {
    return root;
  }

  /**
   * a http server is always started in a new thread, awaiting the connections
   * from the clients.
   */
  public void run() {
    Socket client;
    while (running) {
      try {
        client = ss.accept();
        new HttpServerConnection(client);

      }
      catch (IOException ex) {
      }
    }
  }

  private void jbInit() throws Exception {
  }

  /**
   * <p>Http server connection.</p>
   *
   * <p>Description: this class represents the connection betwwen a particular
   * http client to the server. the connection runs in its own thread, serve the
   * comming http requests. </p>
   *
   * <p>Copyright: Copyright (c) 2005 Jun Hu</p>
   *
   * <p>Company: ID, TU/e </p>
   * @author Jun Hu
   * @version 1.0
   */
  class HttpServerConnection
      implements Runnable {
    final static String CRLF = "\r\n";
    String serverLine = "Server: Jun Hu's httpServer" + CRLF;
    String entityBody = "<HTML>" +
        "<HEAD><TITLE>404 Not Found</TITLE></HEAD>" +
        "<BODY>404 Not Found</BODY></HTML>";
    String statusLine = null;
    String contentLengthLine = null;
    String contentTypeLine = "Content-Type: text/HTML" + CRLF;

    Socket client;

    HttpServerConnection(Socket client) throws SocketException {
      this.client = client;
      (new Thread(this)).start();
    }

    public void run() {
      try {
        BufferedReader in = new BufferedReader(
            new InputStreamReader(client.getInputStream(), "8859_1"));
        OutputStream out = client.getOutputStream();
        PrintWriter pout = new PrintWriter(
            new OutputStreamWriter(out, "8859_1"), true);
        String request = in.readLine();
        System.out.println("Request: " + request);

        Matcher get = Pattern.compile("GET /?(\\S*).*").matcher(request);
        if (get.matches()) {
          request = get.group(1);

          if (request.endsWith("/") || request.equals("")) {
            request = request + "index.html";
          }
          request = URLDecoder.decode(root + request, "UTF-8");
          try {
            InputStream fin = new FileInputStream(new File(request));
            String statusLine = statusLine = "HTTP/1.0 200 OK" + CRLF; ;
            String contentLengthLine = "Content-Length: "
                + (new Integer(fin.available())).toString()
                + CRLF;

            ProgressMonitorInputStream fis = new ProgressMonitorInputStream(null,
                "Sending file : " + request + "...", fin);
            fis.getProgressMonitor().setMillisToDecideToPopup(1000);

            // Send the status line.
            out.write(statusLine.getBytes());

            // Send the server line.
            out.write(serverLine.getBytes());

            // Send the Content-Length
            out.write(contentLengthLine.getBytes());

            // Send a blank line to indicate the end of the header lines.
            out.write(CRLF.getBytes());

            byte[] data = new byte[1024];
            for (int read; (read = fis.read(data)) > -1; ) {
              out.write(data, 0, read);
            }
            out.flush();

          }
          catch (FileNotFoundException e) {
            statusLine = "HTTP/1.0 404 Not Found" + CRLF;
            // Send the status line.
            out.write(statusLine.getBytes());

            // Send the server line.
            out.write(serverLine.getBytes());

            // Send the content type line.
            out.write(contentTypeLine.getBytes());

            // Send a blank line to indicate the end of the header lines.
            out.write(CRLF.getBytes());

            out.write(entityBody.getBytes());

            out.flush();

          }
        }
        else {
          pout.println("400 Bad Request");
        }
        client.close();
      }
      catch (IOException e) {
        System.out.println("I/O error " + e);
      }
    }
  }
}
