/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

//includes
import java.awt.event.*;
import java.text.*;

import javax.swing.*;
import java.awt.*;

import javax.swing.event.*;
import java.util.*;

import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.*;

import org.jivesoftware.smack.Roster;
import org.jivesoftware.smack.RosterEntry;
import org.jivesoftware.smack.RosterGroup;
import org.jivesoftware.smack.XMPPConnection;

import com.valhalla.jbother.*;
import com.valhalla.jbother.menus.BuddyListPopupMenu;
import com.valhalla.jbother.jabber.BuddyStatus;
import com.valhalla.settings.Settings;
import org.dotuseful.ui.tree.*;

/**
 * BuddyListTree is the part of the buddy list dialog that draws the buddies and their
 * groups from your Jabber roster.  It also displays different pictures for different statuses.
 *
 * @author Adam Olsen
 * @version 1.0
 **/
public class BuddyListTree extends JPanel
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private XMPPConnection connection;
	private Roster roster;

	private AutomatedTreeNode root = new AutomatedTreeNode( "Buddies" );
	private AutomatedTreeModel model = new AutomatedTreeModel( root );
	private JTree tree = new JTree( model );

	private JScrollPane scrollPane = new JScrollPane( tree );
	private BuddyListPopupMenu buddyPopupMenu = new BuddyListPopupMenu();
	private boolean showOfflineBuddies = Settings.getInstance().getBoolean( "showOfflineBuddies" );
	private boolean showUnfiledBuddies = Settings.getInstance().getBoolean( "showUnfiledBuddies" );
	private boolean showAgentBuddies = Settings.getInstance().getBoolean( "showAgentBuddies" );
    private boolean showAgentMessages = Settings.getInstance().getBoolean( "showAgentMessages" );
	private BuddyListRenderer renderer = new BuddyListRenderer();
	private BuddyListExpansionListener expandListener = new BuddyListExpansionListener();
	private TreeMap buddyGroups = new TreeMap(); // to sort the buddy groups

	/**
	 * Sets up the tree
	*/
	public BuddyListTree()
	{
		setLayout( new GridLayout( 0, 1 ) );
		setBackground( Color.WHITE );

		tree.setCellRenderer( renderer );
		tree.setRootVisible( false );
		tree.setRowHeight( 0 );
		tree.setShowsRootHandles( true );
		tree.getSelectionModel().setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
		tree.addMouseListener( new PopupMouseListener() );
		tree.addTreeExpansionListener( expandListener );

		ToolTipManager.sharedInstance().registerComponent( tree );
		add( scrollPane );
	}

	/**
	 * Returns the JTree
	 * @return the actual JTree swing component
	*/
	public JTree getTree() { return this.tree; }

	/**
	 * Sets the JTree's XMPPConnection
	 * @param connection the current connection
	*/
	public void setConnection( XMPPConnection connection )
	{
		this.connection = connection;
		if( connection == null ) return;
		this.roster = connection.getRoster();
	}

	/**
	 * Sets whether or not to show the offline buddies
	 * @param show true to show the offline buddies
	*/
	public void setShowOfflineBuddies( boolean show )
	{
		this.showOfflineBuddies = show;
		reloadBuddies();
	}

	/**
	 * Sets whether or not to show the unfiled buddies
	 * @param show true to show unfiled buddies
	*/
	public void setShowUnfiledBuddies( boolean show )
	{
		this.showUnfiledBuddies = show;
		reloadBuddies();
	}

	/**
	 * Sets whether or not to show agents/transports
	 * @param show true to show agents and transports
	*/
	public void setShowAgentBuddies( boolean show )
	{
		this.showAgentBuddies = show;
		reloadBuddies();
	}

	/**
	 * Set to true if you want to show messages from agents
	 * @param show whether or not to recieve messages
	*/
    public void setShowAgentMessages( boolean show )
    {
        this.showAgentMessages = show;
    }

	/**
	 * Returns whether or not offlines buddies are showing
	 * @return true if offline buddies are showing
	*/
	public boolean getShowOfflineBuddies() { return this.showOfflineBuddies; }

	/**
	 * Returns whether or not unfiled buddies are showing
	 * @return true if unfiled buddies are showing
	*/
	public boolean getShowUnfiledBuddies() { return this.showUnfiledBuddies; }

	/**
	 * Returns whether or not agent buddies are showing
	 * @return true if agent buddies are showing
	*/
	public boolean getShowAgentBuddies() { return this.showAgentBuddies; }

	/**
	 * Returns whether or now agent messages are to be recieved
	 * @return true if agent messages are being recieved
	*/
    public boolean getShowAgentMessages() { return this.showAgentMessages; }

	/**
	 * To listen to when a group gets expanded.  Saves it in the settings
	 * for session restoration
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class BuddyListExpansionListener implements TreeExpansionListener
	{
		/**
		 * Listens for a tree collapse event
		*/
		public void treeCollapsed( TreeExpansionEvent e )
		{
			TreePath path = e.getPath();
			AutomatedTreeNode node = (AutomatedTreeNode)path.getLastPathComponent();

			Settings.getInstance().setProperty( "groupExpandStatus_" + node.getUserObject().toString(), "collapsed" );
		}

		/**
		 * Listens for a tree expand event
		*/
		public void treeExpanded( TreeExpansionEvent e )
		{
			TreePath path = e.getPath();
			AutomatedTreeNode node = (AutomatedTreeNode)path.getLastPathComponent();

			Settings.getInstance().remove( "groupExpandStatus_" + node.getUserObject().toString() );
		}
	}

	/**
	 * Redraws the JTree
	*/
	public void reloadBuddies()
	{
		reloadBuddies( false );
	}

	/**
	 * Shows all the current offline buddies
	*/
	public void loadOfflineBuddies()
	{
		reloadBuddies( true );
	}

	/**
	 * Redraws the JTree
	 * @param loadOffline whether or not to just load the offline buddies
	*/
	public void reloadBuddies( final boolean loadOffline )
	{
		SwingUtilities.invokeLater( new Runnable()
		{
			public void run()
			{
				if( connection == null ) return;
				if( roster == null ) roster = connection.getRoster();

				if( !loadOffline ) clearBuddies();

				// loop through all the RosterEntries and see if they need to be added to the
				// BuddyList tree
				Iterator it = roster.getEntries();
				while( it.hasNext() )
				{
					RosterEntry entry = (RosterEntry)it.next();
					BuddyStatus buddy = BuddyList.getInstance().getBuddyStatus( entry.getUser() );

					if( loadOffline && buddy.size() <= 0 ) checkAddEntry( buddy );
					else checkAddEntry( buddy );
				}

				tree.repaint();
			}
		} );
	}

	/**
	 * Clears all the buddies from the JTree
	*/
	public void clearBuddies()
	{
		com.valhalla.Logger.debug( "Clearing Buddies" );
		// clear the JTree and the BuddyGroups TreeMap
		root.removeAllChildren();
		buddyGroups.clear();
	}

	/**
	 * We store all of the Group names in a TreeMap so that we can get the sort
	 * index when inserting the TreeNode for this group into the root node
	 * @param group the name of the group
	**/
	private int getGroupIndex( String group )
	{
		synchronized( buddyGroups )
		{
			if( buddyGroups.containsKey( group ) ) return -1;
			// we want General Contacts and Agents/Transports to always
			// sort at the bottom
			if( group.equals( resources.getString( "contactsGroup" ) ) ) group = "zzz Contacts";
			else if( group.equals( resources.getString( "transportsGroup" ) ) ) group = "zzzz Agents/Transports";

			buddyGroups.put( group, new TreeMap() );

			int count = 0;

			// find the index of the newly sorted group
			Iterator i = buddyGroups.keySet().iterator();
			while( i.hasNext() )
			{
				String key = (String)i.next();
				if( key.equals( group ) ) break;
				count++;
			}

			return count;
		}
	}

	/**
	 * Finds out the sort index of this particular buddy
	 * @param group the group the buddy is in
	 * @param buddy the buddy to find the index of
	**/
	private int getBuddyIndex( String group, BuddyStatus buddy )
	{
		synchronized( buddyGroups )
		{
			if( group.equals( resources.getString( "contactsGroup" ) ) ) group = "zzz Contacts";
			else if( group.equals( resources.getString( "transportsGroup" ) ) ) group = "zzzz Agents/Transports";

			String name = buddy.getName();
			if( name == null ) name = buddy.getUser();
			name = name.toLowerCase();

			if( buddyGroups.get( group ) == null ) buddyGroups.put( group, new TreeMap() );

			((TreeMap)buddyGroups.get( group )).put( name, buddy.getUser() );

			int count = 0;

			Iterator i = ((TreeMap)buddyGroups.get( group )).keySet().iterator();
			while( i.hasNext() )
			{
				String key = (String)i.next();
				if( key.equals( name ) ) break;
				count++;
			}

			return count;
		}
	}

	/**
	 * Checks to see if the group is already in the tree, and returns the index of it
	 * @param group the group to check
	**/
	public AutomatedTreeNode checkGroup( final String group )
	{
		boolean check = false;
		AutomatedTreeNode node = new AutomatedTreeNode( group );

		Enumeration children = root.children();

		// find out if the group alread exists
		whileLoop:
		while( children.hasMoreElements() )
		{
			AutomatedTreeNode theNode = (AutomatedTreeNode)children.nextElement();
			String g = (String)theNode.getUserObject();
			if( g.equals( group ) )
			{
				node = theNode;
				check = true;
				break whileLoop;
			}
		}

		if( root.isNodeChild( node ) ) return node;

		final String tempGroup = group;
		final AutomatedTreeNode tempNode = node;

		if( !check )
		{
			int num = getGroupIndex( tempGroup );
			if( num >= 0 ) root.insert( tempNode, num );
		}

		return node;


	}

	/**
	 * finds out if the buddy should be displayed in the BuddyListTree.
	 * If so the buddy is added to the tree
	 * @param buddy the buddy to add
	**/
	public void checkAddEntry( final BuddyStatus buddy )
	{
		if( buddy == null ) return;

		boolean add = false;

		// if we are set to show the offline buddies then add the buddy to the tree
		if( showOfflineBuddies ) add = true;
		else {
			// otherwise we have to find out if the buddy is online before we add it
			if( buddy.size() > 0 ) add = true;
		}

		if( buddy.getRemoved() ) add = false;

		if( add )
		{
			// find the group that the buddy belongs to

			String tempGroup = buddy.getTempGroup();
			if( tempGroup == null ) tempGroup = buddy.getGroup();

			final String group = tempGroup;

			if( !showUnfiledBuddies && group.equals( resources.getString( "contactsGroup" ) ) ) return;
			if( !showAgentBuddies && group.equals( resources.getString( "transportsGroup" ) ) ) return;

			final AutomatedTreeNode node = checkGroup( group );
			int index = getBuddyIndex( group, buddy );

			if( !isInTree( buddy ) ) node.insert( new AutomatedTreeNode( buddy ),
				index );

			TreePath parent = new TreePath( root );
			tree.expandPath( parent );

			// find out if we need to expand this group
			String property = Settings.getInstance().getProperty( "groupExpandStatus_" + group );
			if( property == null || !property.equals( "collapsed" ) )
			{
				tree.expandPath( parent.pathByAddingChild( node ) );
			}
		}

	}

	/**
	 * Returns whether or not the buddy is in the tree
	 * @param buddy the buddy to check
	 * @return true if the buddy is in the tree
	*/
	public boolean isInTree( BuddyStatus buddy )
	{
		String group = buddy.getGroup();

		// loop through all the groups until we find the group that this
		// buddy belongs to
		Enumeration children = root.children();
		while( children.hasMoreElements() )
		{
			AutomatedTreeNode node = (AutomatedTreeNode)children.nextElement();

			// once we find it's group, loop through all the buddies in that group
			if( node.getUserObject().toString().equals( group ) )
			{
				Enumeration leafs = node.children();
				while( leafs.hasMoreElements() )
				{
					AutomatedTreeNode leaf = (AutomatedTreeNode)leafs.nextElement();
					if( leaf.getUserObject() == buddy ) return true;
				}
			}
		}

		return false;
	}

	/**
	 * Adds a buddy to the tree - if it's not already in the tree
	 * @param buddy the buddy to add
	*/
	public void addBuddy( final BuddyStatus buddy )
	{
		SwingUtilities.invokeLater( new Runnable()
		{
			public void run()
			{
				checkAddEntry( buddy );
				tree.repaint();
				validate();
				repaint();
			}
		} );
	}

	/**
	 * Removes the buddy from the tree
	 * @param buddy the buddy to remove
	 * @param group the group the buddy is in
	*/
	public void removeBuddy( final BuddyStatus buddy, final String group )
	{
		SwingUtilities.invokeLater( new Runnable()
		{
			public void run()
			{
				// loop through all the groups until we find the group that this
				// buddy belongs to
				Enumeration children = root.children();
				while( children.hasMoreElements() )
				{
					AutomatedTreeNode node = (AutomatedTreeNode)children.nextElement();

					// once we find it's group, loop through all the buddies in that group
					if( node.getUserObject().toString().equals( group ) )
					{
						Enumeration leafs = node.children();
						while( leafs.hasMoreElements() )
						{
							AutomatedTreeNode leaf = (AutomatedTreeNode)leafs.nextElement();
							Object check = leaf.getUserObject();
							if( check instanceof BuddyStatus )
							{
								BuddyStatus temp = (BuddyStatus)check;

								if( temp.getUser().equals( buddy.getUser() ) )
								{
									// once we find the buddy, remove it
									removeBuddyNode( node, leaf );
								}
							}
						}
					}
				}
			}
		} );
	}

	/**
	 * removes this buddy from the JTree.  If this was the last buddy in the group
	 * then remove the group from being displayed
	 * @param node the group node
	 * @param leaf the leaf node
	**/
	private void removeBuddyNode( final AutomatedTreeNode node, final AutomatedTreeNode leaf )
	{
		String group = (String)node.getUserObject();

		if( group.equals( resources.getString( "contactsGroup" ) ) ) group = "zzz Contacts";
		else if( group.equals( resources.getString( "transportsGroup" ) ) ) group = "zzzz Agents/Transports";

		BuddyStatus buddy = (BuddyStatus)leaf.getUserObject();

		String name = buddy.getName();
		if( name == null ) name = buddy.getUser();
		String jid = buddy.getUser();

		synchronized (buddyGroups) {
			TreeMap buddies = ((TreeMap) buddyGroups.get(group));
			Iterator i = buddies.keySet().iterator();
			while (i.hasNext()) {
				String key = (String) i.next();
				if (buddies.get(key).equals(jid)) {
					((TreeMap) buddyGroups.get(group)).remove(key);
					break;
				}
			}

		}

		node.remove( leaf );
		if( node.getChildCount() <= 0 )
		{
			buddyGroups.remove( group );
			root.remove( node );
		}

		tree.repaint();
	}

	/**
	 * Starts a conversation if someone double double clicks on a buddy
	*/
	public void initiateConversation()
	{
		if( tree.getSelectionPath() == null ) return;
		TreePath path = tree.getSelectionPath();
		AutomatedTreeNode node = (AutomatedTreeNode)path.getLastPathComponent();
		if( model.isLeaf( node ) )
		{
			BuddyStatus buddy = (BuddyStatus)node.getUserObject();

			if( buddy.getConversation() == null )
			{
				ChatPanel conver = new ChatPanel( buddy );
				buddy.setConversation( conver );
				MessageDelegator.getInstance().showPanel( buddy.getConversation() );
				MessageDelegator.getInstance().frontFrame( buddy.getConversation() );
			}
			else {
				MessageDelegator.getInstance().showPanel( buddy.getConversation() );
				MessageDelegator.getInstance().frontFrame( buddy.getConversation() );
			}

			buddy.getConversation().stopTimer();
		}
	}

	/**
	 * Listens for mouse events in the tree
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class PopupMouseListener extends MouseAdapter
	{
		public void mousePressed( MouseEvent e ) { checkPop( e ); }
		public void mouseReleased( MouseEvent e ) { checkPop( e ); }
		public void mouseClicked( MouseEvent e )
		{
			checkPop( e );
			try {
				JTree tree = (JTree)e.getComponent();
			}
			catch( ClassCastException ex ) { return; }

			if( e.getClickCount() >= 2 )
			{
				initiateConversation();
			}
		}

		/**
		 * Checks if we need to display the buddy list popup menu
		 * Shows it if needs be
		*/
		public void checkPop( MouseEvent e )
		{
			BuddyStatus buddy = null;

			if( e.isPopupTrigger() )
			{
				try {

					JTree tree = (JTree)e.getComponent();

					TreePath path = tree.getPathForLocation( e.getX(), e.getY() );
					if( path == null ) 	throw new ClassCastException();

					tree.setSelectionPath( path );

					AutomatedTreeNode node = (AutomatedTreeNode)path.getLastPathComponent();
					buddy = (BuddyStatus)node.getUserObject();
					buddyPopupMenu.showMenu( e.getComponent(), e.getX(), e.getY(), buddy );
				}
				catch( ClassCastException ex ) { /*is not a buddy, so don't display the menu*/ }
			}
		}
	}
}
