/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import javax.swing.SwingUtilities;
import java.awt.event.*;

import org.jivesoftware.smack.*;
import org.jivesoftware.smack.filter.*;
import org.jivesoftware.smack.packet.*;
import org.jivesoftware.smackx.packet.*;

import java.util.*;
import java.lang.reflect.*;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.smack.*;
import org.jivesoftware.smack.provider.ProviderManager;

import com.valhalla.jbother.jabber.smack.provider.*;
import com.valhalla.jbother.jabber.*;
import com.valhalla.settings.Settings;

/**
 * Attempts to connect to the server.
 * If the connection is made successfully, it sets up the various packet listeners
 * and displays the BuddyList
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class ConnectorThread implements Runnable
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private String server, username, password, resource;
	private boolean ssl;
	private int port = 0;
	private String errorMessage = null;
	private XMPPConnection connection = null;
	private static boolean hasHadError = false;
	private static com.valhalla.jbother.jabber.smack.ConnectionListener conListener =
		new com.valhalla.jbother.jabber.smack.ConnectionListener();
	private static com.valhalla.jbother.jabber.smack.RosterListener rosterListener =
		new com.valhalla.jbother.jabber.smack.RosterListener();
	private Presence.Mode connectMode = Presence.Mode.AVAILABLE;
	private String statusString = null;
	private boolean persistent = false;
	private static boolean cancelled = false;
	private boolean away = false;

	/**
	 * Sets up the connector thread
	 * @param box the Object that contains all the required login information
	*/
	public ConnectorThread( Presence.Mode connectMode, String statusString, boolean away )
	{
		this.server = Settings.getInstance().getProperty( "defaultServer" );
		this.username = Settings.getInstance().getProperty( "username" );
		this.password = Settings.getInstance().getProperty( "password" );
		this.resource = Settings.getInstance().getProperty( "resource" );
		this.ssl = Settings.getInstance().getBoolean( "useSSL" );

		String p = Settings.getInstance().getProperty( "port" );
		if( p != null )
		{
			try {
				port = Integer.parseInt( p );
			}
			catch( NumberFormatException ex ) { }
		}

		hasHadError = false;
		this.connectMode = connectMode;
		this.statusString = statusString;
		this.away = away;
	}

	public static void setCancelled( boolean c )
	{
		cancelled = c;
	}

	/**
	 * Sets whether or not this thread should try to reconnect if there is a connection error
	 *
	 * @param perisisten set to <tt>true</tt> if you want the thread to continue to try and connect even if there's an error
	*/
	public void setPersistent( boolean persistent ) { this.persistent = persistent; }

	/**
	 * Returns the ConnectionListener
	 * @return the connection listener
	*/
	public static com.valhalla.jbother.jabber.smack.ConnectionListener getConnectionListener() { return conListener; }

	/**
	 * Sets whether or not a connection error has already been thrown for this connection
	 * @param has true if an error has already been thrown
	**/
	public static void setHasHadError( boolean has ) { hasHadError = has; }

	/**
	 * Called when the Threads .start() method is called
	*/
	public void run()
	{
		if( !BuddyList.getInstance().isVisible() ) BuddyList.getInstance().setVisible( true );
		if( !BuddyList.getInstance().getTopMenu().getStatusMenu().blinkTimerIsRunning() )
		{
			BuddyList.getInstance().getTopMenu().getStatusMenu().startBlinkTimer();
		}

		if( cancelled )
		{
			BuddyList.getInstance().getTopMenu().getStatusMenu().stopBlinkTimer();
			cancelled = false;
			return;
		}

		try {
			if( com.valhalla.settings.Arguments.getInstance().getProperty( "smackdebug" ) != null ) XMPPConnection.DEBUG_ENABLED = true;

			int port = this.port;
			if( port == 0 && ssl ) port = 5223;
			else if( port == 0 && !ssl ) port = 5222;

			// set up the connection
			if( !ssl ) connection = connection = new XMPPConnection( server, port );
			else connection = new SSLXMPPConnection( server, port );
			BuddyList.getInstance().clearTree();
			BuddyList.getInstance().init( connection );

		}
		catch( Exception ex )
		{
			errorMessage = ex.getMessage();
		}

		if( cancelled )
		{
			BuddyList.getInstance().getTopMenu().getStatusMenu().stopBlinkTimer();
			cancelled = false;

			return;
		}

		// get the resource from the login box
		String tmp = resource;
		if( tmp == null || tmp.equals( "" ) ) tmp = "JBother";
		final String resource = tmp;

		if( errorMessage == null && connection != null )
		{
			// sets up the various packet listeners
			PacketFilter filter = new PacketTypeFilter( Presence.class );
			connection.addPacketListener( new PresencePacketListener(), filter );
			filter = new PacketTypeFilter( Message.class );
			connection.addPacketListener( new MessagePacketListener(), filter );
			connection.addConnectionListener( conListener );
			filter = new PacketTypeFilter( Version.class );
			connection.addPacketListener( new VersionListener(), filter );
			filter = new PacketTypeFilter( Time.class );
			connection.addPacketListener( new TimeListener(), filter );

			ProviderManager.addIQProvider( "query", "jabber:iq:version", new VersionProvider() );
			ProviderManager.addIQProvider( "query", "jabber:iq:time", new TimeProvider() );

			// attempts to connect
			try {
				connection.login( username, password, resource );
				connection.getRoster().setSubscriptionMode( Roster.SUBSCRIPTION_MANUAL );
				connection.getRoster().addRosterListener( rosterListener );
			}
			catch( XMPPException e )
			{
				errorMessage = e.getMessage();
			}

		}

		// if there was an error, display it, and then redisplay a LoginDialog
		if( errorMessage != null || connection == null )
		{
			BuddyList.getInstance().getTopMenu().getStatusMenu().setModeChecked( null );

			if( persistent )
			{
				try {
					Thread.sleep( 3000 );
				}
				catch( InterruptedException ex ) { }
				errorMessage = null;

				if( cancelled )
				{
					cancelled = false;

					BuddyList.getInstance().getTopMenu().getStatusMenu().stopBlinkTimer();
					return;
				}

				run();
				return;
			}

			BuddyList.getInstance().getTopMenu().getStatusMenu().stopBlinkTimer();


			if( errorMessage == null ) errorMessage = resources.getString( "connectionError" );
			if( errorMessage.equals( "Unauthorized" ) )
					errorMessage = new String( resources.getString( "invalidPassword" ) );

			Standard.warningMessage( null, resources.getString( "couldNotConnect" ), errorMessage );

			return;
		}

		BuddyList.getInstance().getTopMenu().getStatusMenu().stopBlinkTimer();

		// otherwise, set up and display the buddy list
		com.valhalla.Logger.debug( "Connected" );

		BuddyList.getInstance().startTimer();
		BuddyList.getInstance().getTopMenu().getStatusMenu().setModeChecked( connectMode );
		BuddyList.getInstance().initBuddies(); // display the buddies
		BuddyList.getInstance().setStatus( connectMode, statusString, false );

		if( away )
		{
			BuddyList.getInstance().getAwayHandler().actionPerformed( new ActionEvent( BuddyList.getInstance(), 1, "away" ) );
		}

		return;
	}
}
