/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package com.valhalla.jbother;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.regex.Pattern;

import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.StyleSheet;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.settings.Settings;

/**
 * Represents an appending JEditorPane.
 * Text appended to this pane will cause it's scrollbar to scroll to the bottom.
 * Also handles hyperlink events
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class ConversationArea extends JEditorPane implements HyperlinkListener
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JScrollPane scroll;
	private HTMLEditorKit kit = (HTMLEditorKit)getEditorKit();
	private HTMLDocument doc = (HTMLDocument)getDocument();
	private String lastHTML;
	private javax.swing.Timer timer = new javax.swing.Timer( 100, new ShowHandler() );
	private StringBuffer buffer = new StringBuffer();
	private StyleSheet myStyleSheet = new StyleSheet();

	/**
	 * Default constructor - sets up the JEditorPane
	 */
	public ConversationArea()
	{
		super( "text/html; enctype='UTF-8'", "" );

		String fontString = Settings.getInstance().getProperty( "messageWindowFont" );
		Font windowFont;
		if( fontString == null ) windowFont = new Font( "Sans", Font.PLAIN, 12 );
		else windowFont = Font.decode( fontString );

		loadStyleSheet( windowFont );

		kit.setStyleSheet( myStyleSheet );
		doc = new HTMLDocument( myStyleSheet );
		setDocument( doc );
		setEditable( false );

		addHyperlinkListener( this );
		setBackground( Color.WHITE );
	}

	/**
	 * Loads a new font for the JEditorPane
	 * @param windowFont the font to load
	*/
	public void loadStyleSheet( Font windowFont )
	{
		String ruleString = "font-size: " + windowFont.getSize() + "; font-family: " + windowFont.getName() + "; ";
		if( windowFont.isItalic() ) ruleString += "font-style: italic; ";
		else ruleString += "font-style: normal; ";

		if( windowFont.isBold() ) ruleString += "font-weight: bold; ";
		else ruleString += "font-weight: normal; ";

		myStyleSheet.addRule( "body { " + ruleString + " }" );
		myStyleSheet.addRule( "body a { color: #5B62BC; text-decoration: underline }" );
	}

	/**
	 * Sets this window's scrollbar
	 * @param scroll the scrollbar
	*/
	public void setScroll( JScrollPane scroll )
	{
		this.scroll = scroll;
	}

	/**
	 * Appends text to the ConversationArea
	 * Scrolls down if the scrollbar is already at the bottom, or
	 * does nothing if the person has scrolled up any
	 * @param html the text to append
	*/
	public void append( String html )
	{
		html = html.replaceAll( "\\n", "<br>" );
		lastHTML = html;
		buffer.append( html );
		if( !timer.isRunning() ) timer.start();
		else timer.restart();
	}

	/**
	 * Actually appends the buffered data to the JEditorPane
	 * @author Adam Olsen
	 * @version 2.0
	*/
	class ShowHandler implements ActionListener
	{
		public synchronized void actionPerformed( ActionEvent ae )
		{
			JScrollBar bar = scroll.getVerticalScrollBar();

			boolean end = bar.getValue() - ( bar.getMaximum() - bar.getModel().getExtent() ) >= -16;
			Point p = scroll.getViewport().getViewPosition();

			boolean scrFlag = bar.isVisible();
			p.y += 50;  // just so it's not the first line (might scroll a bit)
			int pos = viewToModel( p );

			if( pos < 0 ) pos = 0;

			try {

				kit.insertHTML( doc, doc.getLength(), buffer.toString().replaceAll( "\\<br\\>$", "" ), 0, 0, null );

				if( !end && scrFlag ) setCaretPosition( pos );
				else setCaretPosition( doc.getLength() );
				scroll.repaint();
			}
			catch( Exception e )
			{
				com.valhalla.Logger.debug( "ShowHandler.class Exception: " + e.toString() );
			}

			buffer = new StringBuffer();
			timer.stop();
		}
	}

	/**
	 * @return the last appended HTML
	*/
	public String getLastHTML() { return this.lastHTML; }

	/**
	 * Moves the cursor to the end of the editorpane
	 */
	public void moveToEnd()
	{
		try {
			setCaretPosition( doc.getLength() );
		}
		catch( Exception e ) { }
	}

	/**
	 * Is called when someone clicks on a hyperlink in the window
	**/
	public void hyperlinkUpdate( HyperlinkEvent e )
	{
		if( e.getEventType() == HyperlinkEvent.EventType.ACTIVATED )
		{
			try {
				String app = Settings.getInstance().getProperty( "browserApplication" );
				if( Pattern.matches( "^mailto\\:.*", e.getDescription() ) ) app = Settings.getInstance().getProperty( "emailApplication" );

				if( app != null && !app.equals( "" ) ) {
					/*tail*/
                    String command = app.replaceAll( "%s", e.getDescription() );
                    command = command.replaceAll( "\\%l", e.getDescription() ); // for backwards compatability

                    Runtime.getRuntime().exec( command );
					/*tail*/
                }
				else Standard.warningMessage( this, resources.getString( "hyperlink" ),
					resources.getString( "noApplication" ) );
			}
			catch( java.io.IOException ex ) { Standard.warningMessage( this, resources.getString( "hyperlink" ),
				resources.getString( "errorExecutingApplication" ) ); }
		}
	}
}
