/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import com.valhalla.jbother.jabber.*;
import com.valhalla.jbother.jabber.smack.*;

import java.util.*;
import org.jivesoftware.smack.*;
import org.jivesoftware.smack.packet.*;
import org.jivesoftware.smackx.packet.*;
import org.jivesoftware.smack.filter.*;
import com.valhalla.gui.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import javax.swing.border.*;
import com.valhalla.jbother.*;

/**
 * A dialog that collects and shows information about a Jabber user
 * Shows a dialog with several fields and starts an information collecting field for each one.
 * As each piece of information is found it fills out the fields
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class InformationViewerDialog extends JDialog
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private ArrayList values = new ArrayList();
	private JButton okButton = new JButton( resources.getString( "okButton" ) );
	private String user;
	private JPanel mainPanel;
	private JPanel rightInner = new JPanel();
	private JPanel leftInner = new JPanel();
	private JPanel innerPanel = new JPanel();
	private InformationViewerDialog thisPointer = this;

	/**
	 * Default constructor
	 * @param user the user you want to collect information about
	 */
	public InformationViewerDialog( String user )
	{
		this.user = user;
		setTitle( resources.getString( "information" ) + " " + user );

		mainPanel = (JPanel)getContentPane();
		innerPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
		innerPanel.setLayout( new BoxLayout( innerPanel, BoxLayout.X_AXIS ) );

		rightInner = new JPanel();
		rightInner.setLayout( new BoxLayout( rightInner, BoxLayout.Y_AXIS ) );
		leftInner = new JPanel();
		leftInner.setLayout( new BoxLayout( leftInner, BoxLayout.Y_AXIS ) );
		innerPanel.add( leftInner );
		innerPanel.add( rightInner );

		mainPanel.setLayout( new BorderLayout() );
		mainPanel.setBorder( BorderFactory.createTitledBorder( user ) );
		mainPanel.add( innerPanel, BorderLayout.CENTER );

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout( new BoxLayout( buttonPanel, BoxLayout.X_AXIS ) );
		buttonPanel.add( Box.createHorizontalGlue() );
		buttonPanel.add( okButton );
		buttonPanel.add( Box.createHorizontalGlue() );
		buttonPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

		mainPanel.add( buttonPanel, BorderLayout.SOUTH );

		addItems();
		pack();

		Dimension dim = getSize();
		setSize( new Dimension( 500, (int)dim.getHeight() ) );

		setLocationRelativeTo( null );
		DialogTracker.addDialog( this, false, true );

		okButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				DialogTracker.removeDialog( thisPointer );
			}
		} );
	}

	/**
	 * Adds the different fields to the displayed form
	*/
	private void addItems()
	{
		/* gets jabber:iq:version information */
		addItem( resources.getString( "clientInformation" ), VersionCollector.class );
		addItem( resources.getString( "timeInformation" ), TimeCollector.class );
	}

	/**
	 * Adds a field and starts the thread that will collect it's information
	 * @param key the name of the field
	 * @param informationThread the thread that will collect the information
	 */
	private void addItem( String key, Class informationThread )
	{
		UIDefaults ui = UIManager.getDefaults();
		Font newFont = (Font)ui.get( "Label.font" );

		JLabel label = new JLabel( key + ":  " );
		label.setFont( new Font( newFont.getName(), Font.BOLD, newFont.getSize() ) );
		leftInner.add( label );

		JLabel valueLabel = new JLabel( resources.getString( "fetchingInfo" ) );
		rightInner.add( valueLabel );

		try {
			// start the information collector thread
			Class[] thisArgsClass = new Class[] { InformationViewerDialog.class, Integer.class };
			Object[] args = new Object[] { this, new Integer( values.size() ) };

			Constructor con = informationThread.getConstructor( thisArgsClass );
			InformationCollector col = (InformationCollector)con.newInstance( args );
			Thread thread = new Thread( col );
			thread.start();
		}
		catch( Exception e )
		{
			com.valhalla.Logger.logException( e );
		}

		values.add( valueLabel );
	}

	/**
	 * Gets the different field names
	 * @return The ArrayList containing all the field names
	*/
	protected ArrayList getValues() { return values; }

	/**
	 * Gets the user that the InformationDialog was instantiated for
	 * @return the user
	*/
	protected String getUser() { return user; }
}

/**
 * Abstract information thread
 * Each subclass of this class is to collect the information for a specific field
 * @author Adam Olsen
 * @version 1.0
*/
abstract class InformationCollector implements Runnable
{
	protected Integer id;
	protected InformationViewerDialog dialog;

	/**
	 * The default constructor
	 * @param dialog the InformationViewerDialog that called this thread
	 * @param id the field id
	*/
	public InformationCollector( InformationViewerDialog dialog,  Integer id ) { this.dialog = dialog; this.id = id; }

	/**
	 * Called by the enclosing <code>java.util.Thread</code>
	*/
	abstract public void run();
}

/**
 * An <code>InformationCollector</code> that collects jabbber:iq:version information
 * @author Adam Olsen
 * @version 1.0
*/
class VersionCollector extends InformationCollector
{
	/**
	 * The default constructor
	 * @param dialog the InformationViewerDialog that called this thread
	 * @param id the field id
	*/
	public VersionCollector( InformationViewerDialog dialog, Integer id ) { super( dialog, id ); }

	/**
	 * Called by the enclosing <code>java.util.Thread</code>
	 * The run() method is responsible for collecting the requested information
	 * and setting the field in the InformationViewerDialog
	*/
	public void run()
	{
		XMPPConnection con = BuddyList.getInstance().getConnection();

		Version request = new Version();
		request.setType( IQ.Type.GET );
		request.setTo( dialog.getUser() );

		// Create a packet collector to listen for a response.
		PacketCollector collector = con.createPacketCollector(
			new PacketIDFilter( request.getPacketID() ) );

		con.sendPacket( request );

		// Wait up to 5 seconds for a result.
		IQ result = (IQ)collector.nextResult( 3000 );
		JLabel label = (JLabel)dialog.getValues().get( id.intValue() );

		if( result != null && result.getType() == IQ.Type.RESULT )
		{
			Version v = (Version)result;

			label.setText( v.getName() + " " + v.getVersion() + " / " + v.getOs() );
		}
		else {
			label.setText( "N/A" );
		}

		label.repaint();
		dialog.validate();
	}
}

/**
 * @author Adam Olsen
 * @version 1.0
*/
class TimeCollector extends InformationCollector
{
	/**
	 * The default constructor
	 * @param dialog the InformationViewerDialog that called this thread
	 * @param id the field id
	*/
	public TimeCollector( InformationViewerDialog dialog, Integer id ) { super( dialog, id ); }

	/**
	 * Called by the enclosing <code>java.util.Thread</code>
	 * The run() method is responsible for collecting the requested information
	 * and setting the field in the InformationViewerDialog
	*/
	public void run()
	{
		XMPPConnection con = BuddyList.getInstance().getConnection();

		Time request = new Time();
		request.setType( IQ.Type.GET );
		request.setTo( dialog.getUser() );

		// Create a packet collector to listen for a response.
		PacketCollector collector = con.createPacketCollector(
			new PacketIDFilter( request.getPacketID() ) );

		con.sendPacket( request );

		// Wait up to 5 seconds for a result.
		IQ result = (IQ)collector.nextResult( 3000 );
		JLabel label = (JLabel)dialog.getValues().get( id.intValue() );

		if( result != null && result.getType() == IQ.Type.RESULT )
		{
			Time t = (Time)result;

			label.setText( t.getDisplay() );
		}
		else {
			label.setText( "N/A" );
		}

		label.repaint();
		dialog.validate();
	}
}
