/*
 *  Copyright (C) 2003 Adam Olsen
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package com.valhalla.jbother;

import java.io.*;

import javax.swing.*;
import java.awt.*;
import javax.swing.plaf.*;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.settings.*;
import java.util.*;
import java.text.*;
import com.valhalla.pluginmanager.*;

/**
 * Sets default Settings (if this is the first run of JBother), loads command line arguments and
 * settings from the settings file, sets the L&F.
 *
 * @author     Adam Olsen (arolsen@gmail.com)
 * @created    November 30, 2004
 * @version    1.0
 */
public class JBotherLoader
{
	private static PluginLoader loader = PluginLoader.getInstance();

	/**
	 * This is the main class, it basically just provides a loading point for the login
	 * screen - and also allows arguments to be passed from the command line.
	 *
	 * @param  args  arguments passed via the command line
	 * @see          com.valhalla.settings.Arguments
	 */
	public static void startJBother( String args[] )
	{
		// initialize the argument holder
		Arguments.setArguments( args );

		if( System.getProperty( "mrj.version" ) != null )
		{
			System.setProperty( "apple.laf.useScreenMenuBar", "true" );
			System.setProperty( "com.apple.mrj.application.apple.menu.about.name", "JBother" );
		}

		String profile = ProfileManager.getDefaultProfile();
		if( profile == null || profile.equals( "" ) )
		{
			profile = "default";
		}

		JBother.profileDir = JBother.settingsDir + File.separatorChar + "profiles" + File.separatorChar + profile;

		Settings.getInstance().loadSettings( JBother.profileDir, "settings.properties" );
		loadSettings();

		// does any Jbother version change tasks
		JBVersionManager.check();

		com.valhalla.Logger.setLogFile( JBother.settingsDir + File.separatorChar + "jbother.log" );

		if( Arguments.getInstance().getProperty( "webstart" ) == null )
		{
			loadPlugins();
		}

		if( Settings.getInstance().getProperty( "username" ) == null ||
				Settings.getInstance().getProperty( "password" ) == null )
		{
			ProfileEditorDialog dialog = new ProfileEditorDialog( null, profile );
			dialog.setExitOnClose( true );
			dialog.getDefaultBox().setSelected( true );
			dialog.show();

			return;
		}

		if( Arguments.getInstance().getProperty( "prof" ) != null )
		{
			ProfileManager m = new ProfileManager();
			m.setExitOnClose( true );
		}
		else
		{
			launch();
		}
	}

	/**
	 *  Loads some JBother settings
	 */
	public static void loadSettings()
	{
		loadLAF();

		String fontString = Settings.getInstance().getProperty( "applicationFont" );
		if( fontString == null )
		{
			fontString = "Default-PLAIN-12";
		}

		Font newFont = Font.decode( fontString );

		setupFont( newFont );
	}

	/**
	 * Loads the Look And Feel requested in the settings
	 */
	public static void loadLAF()
	{
		String laf = Settings.getInstance().getProperty( "lookAndFeel" );
		if( Arguments.getInstance().getProperty( "laf" ) != null )
		{
			laf = Arguments.getInstance().getProperty( laf );
		}

		UIManager.put( "ClassLoader", loader );
		if( laf != null && Arguments.getInstance().getProperty( "notheme" ) == null )
		{
			try
			{
				Class lafClass = loader.loadClass( laf );
				UIManager.setLookAndFeel( (LookAndFeel)lafClass.newInstance() );
			}
			catch( Exception e )
			{
				com.valhalla.Logger.debug( "Could not load look and feel settings.\n" + e.getMessage() );
			}
		}
	}

	/**
	 * Finds the available plugins and uses the PluginLoader to load them
	 * Once a plugin is loaded, it's init() method is called so it can
	 * execute initial code and register for various events in JBother
	 */
	public static void loadPlugins()
	{
		ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );

		loader.findPlugins( JBother.settingsDir + File.separatorChar + "plugins" );
		loader.loadPlugins();

		ArrayList invalids = loader.getInvalidPlugins();
		for( int i = 0; i < invalids.size(); i++ )
		{
			String name = (String)invalids.get( i );
			Standard.warningMessage( null, resources.getString( "pluginError" ),
					MessageFormat.format( resources.getString( "pluginErrorMessage" ), new Object[]{name} ) );
		}
	}

	/**
	 * Sets the font for the entire application
	 *
	 * @param  font  the font to use
	 */
	public static void setupFont( Font font )
	{
		LookAndFeel lf = javax.swing.UIManager.getLookAndFeel();
		UIDefaults uid = lf.getDefaults();
		Enumeration k = uid.keys();
		while( k.hasMoreElements() )
		{
			Object key = k.nextElement();
			Object val = javax.swing.UIManager.get( key );
			if( val instanceof FontUIResource )
			{
				FontUIResource fuir = (FontUIResource)val;
				javax.swing.UIManager.put( key, new FontUIResource( font ) );
			}
		}
	}

	/**
	 * launches the profile defined as default
	 */
	public static void launch()
	{
		String profile = ProfileManager.getDefaultProfile();
		if( profile == null || profile.equals( "" ) )
		{
			profile = "default";
		}
		ProfileManager.loadProfile( profile );
	}
}

