/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.WindowConstants;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.settings.Settings;

/**
 * Allows the user to view the log of any contact
 * Displays a dialog that allows you to view the log of any Jabber
 * user you have contacted.  You can view the log by date and time
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class LogViewerDialog extends JFrame
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private String logDirectory;
	private JList logList;
	private JPanel container = new JPanel();
	private JPanel rightPanel = new JPanel();
	private ConversationArea logView;
	private File logDirectoryFile;
	private JButton  closeButton = new JButton( resources.getString( "closeButton" ) ),
					 clearButton = new JButton( resources.getString( "clearButton" ) );

	private JSplitPane splitPane;
	private File fileList[];
	private LogViewerCaller caller;
	private LogViewerDialog thisPointer = this;

	/**
	 * Default constructor
	 * Can be passed a ChatRoomPanel, HeadlineWindow, and a ChatPanel as the caller
	 * @param caller the object that called this dialog
	 * @param entryName the user who's log is to be viewed
	*/
	public LogViewerDialog( LogViewerCaller caller, String entryName )
	{
		super( "Log Viewer (" + entryName + ")" );
		setTitle( resources.getString( "logViewer" ) + " (" + entryName + ")" );

		ImageIcon icon = StatusIconCache.getStatusIcon(org.jivesoftware.smack.packet.Presence.Mode.AVAILABLE);
		if ( icon != null ) setIconImage( icon.getImage() );

		if( caller != null ) this.caller = caller;

		logDirectory = JBother.profileDir + File.separatorChar + "logs" + File.separatorChar +
			entryName.replaceAll( "@", "_at_" ).replaceAll( "\\/", "-" );

		logDirectoryFile = new File( logDirectory );

		container.setLayout( new BoxLayout( container, BoxLayout.X_AXIS ) );
		setContentPane( container );

		logView = new ConversationArea();

		logView.setPreferredSize( new Dimension( 500, 350 ) );

		rightPanel.setLayout( new BoxLayout( rightPanel, BoxLayout.Y_AXIS ) );
		rightPanel.add( new JScrollPane( logView ) );

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout( new BoxLayout( buttonPanel, BoxLayout.X_AXIS ) );
		buttonPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 0, 0 ) );
		buttonPanel.add( Box.createHorizontalGlue() );
		buttonPanel.add( clearButton );
		buttonPanel.add( closeButton );
		rightPanel.add( buttonPanel );

		splitPane = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT, new JScrollPane( loadLogFiles() ), rightPanel );
		splitPane.setDividerLocation( 150 );
		splitPane.setOneTouchExpandable( true );

		container.add( splitPane );
		container.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

		setUpListeners();

		pack();
		Standard.cascadePlacement( this );
		DialogTracker.addDialog( this, false, true );

		if( logList.getModel().getSize() >= 1 ) setVisible( true );
		else {
			Standard.warningMessage( null, resources.getString( "logViewer" ),
				resources.getString( "noLogData" ) );
			DialogTracker.removeDialog( this );
		}
	}

	/**
	 * Returns a formatted date string that is used in many places in JBother
	 * @return the formatted date and time
	*/
	public static String getDateName()
	{
		SimpleDateFormat formatter = new SimpleDateFormat( "yyyy-MM-dd-HH-mm-ss" );
		String date = formatter.format( new Date() );
		return date;
	}

	/**
	 * Sets up the listeners for the various events
	*/
	private void setUpListeners()
	{
		setDefaultCloseOperation( WindowConstants.DISPOSE_ON_CLOSE );

		closeButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				DialogTracker.removeDialog( thisPointer );
			}
		} );

		clearButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				clearHandler();
			}
		} );

		logList.addMouseListener( new LogListMouseListener() );
	}

	/**
	 * Listens for someone to click on an entry date
	 * and populates the textArea with the log data
	 * @author Adam Olsen
	 * @version 1.0
	*/
	private class LogListMouseListener extends MouseAdapter
	{
		public void mouseClicked( MouseEvent e )
		{
			File file = (File)logList.getSelectedValue();
			String fileText = "";

			try {
				// open the selected log
				FileReader fr = new FileReader( file );
				BufferedReader in = new BufferedReader( fr );
				String line;
				while( ( line = in.readLine() ) != null ) fileText += line;
				fr.close();
			}
			catch( FileNotFoundException fnfe ) {  }
			catch( IOException ioe ) {  }

			logView.setText( fileText );
		}
	}

	/**
	 * Clears all log data (deletes all log entries)
	*/
	private void clearHandler()
	{
		int result = JOptionPane.showConfirmDialog( null,
			resources.getString( "sureClearLog" ), resources.getString( "clearLogs" ), JOptionPane.YES_NO_OPTION );

		if( result == 0 )
		{
			if( caller != null )
			{
				caller.closeLog();
				caller.startLog();
			}

			for( int i = 0; i < fileList.length; i++ ) fileList[i].delete();
			DialogTracker.removeDialog( this );
		}
	}

	/**
	 * Returns a list of all available log entries
	 * @return the list of log entries
	*/
	private JList loadLogFiles()
	{
		logList = new JList();
		logList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		logList.setCellRenderer( new LogListRenderer() );

		if( !logDirectoryFile.isDirectory() ) return logList;

		// opens the directory and lists the files
		fileList = logDirectoryFile.listFiles( new LogFileFilter() );
		if( fileList.length == 0 ) return logList;

		String fileText = "";

		Arrays.sort( fileList, Collections.reverseOrder() );

		// displays the most recent entry
		try {
			FileReader fr = new FileReader( fileList[0] );
			BufferedReader in = new BufferedReader( fr );
			String line;
			while( ( line = in.readLine() ) != null ) fileText += line;
			fr.close();
		}
		catch( FileNotFoundException fnfe ) { return new JList(); }
		catch( IOException ioe ) { return new JList(); }

		logView.setText( fileText );

		logList.setListData( fileList );
		logList.setSelectedIndex( 0 );
		return logList;
	}
}

/**
 * Displays each log entry as a date and time
 * @author Adam Olsen
 * @version 1.0
*/
class LogListRenderer extends JLabel implements ListCellRenderer
{
	/**
	 * Sets the background to opaque
	*/
	public LogListRenderer()
	{
		setOpaque( true );
	}

	/**
	 * Renders each entry
	*/
	public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSelected, boolean cellHasFocus )
	{
		File file = (File)value;

		String dateString[] = file.toString().replaceAll( ".*\\" + File.separatorChar ,
			"" ).replaceAll( "\\.log\\.html", "" ).split( "\\-" );

		try {
			setText( dateString[1] + "-" + dateString[2] + "-" + dateString[0] + " " + dateString[3] + ":" + dateString[4] + ":" + dateString[5] );
		}
		catch( Exception e )
		{
			setText( "Invalid Date" );
		}

		setBackground( isSelected ? list.getSelectionBackground() : list.getBackground() );
		setForeground( isSelected ? list.getSelectionForeground() : list.getForeground() );
		list.validate();

		return this;
	}
}

/**
 * A filter to only accept html files
 * @author Adam Olsen
 * @version 1.0
*/
class LogFileFilter implements FileFilter
{
	// accept html files
	public boolean accept( File f )
	{
		if( f.isDirectory() ) return false;

		if( f.length() <= 0.0 ) return false;

		String extension = Utils.getExtension( f );

		if( extension != null )
		{
			if( extension.equals( Utils.html ) )
			{
					return true;
			}
			else {
				return false;
			}
		}

		return false;
	}
}
