/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import java.awt.*;
import java.util.regex.Pattern;
import java.util.*;
import org.jivesoftware.smack.packet.*;
import org.jivesoftware.smack.*;

import javax.swing.*;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;

/**
 * Special <code>RegistrationForm</code> that allows you to register for a jabber account
 * @author Adam Olsen
 * @version 1.0
*/
public class NewAccountDialog extends RegistrationForm
{
	private NewAccountDialog thisPointer = this;
	private XMPPConnection connection;
	private AccountManager manager;
	private String username, password;
	private boolean ssl = false;
	private int port = 5222;
	private ProfileEditorDialog profDialog = null;

	/**
	 * Default constructor
	 * @param dialog the <code>LoginDialog</code> that called this form
	 * @param server the server to register for
	*/
	public NewAccountDialog( ProfileEditorDialog dialog, String server, String username, String password, int port, boolean useSSL )
	{
		super( server );

		this.port = port;
		this.ssl = useSSL;
		this.username = username;
		this.password = password;
		this.profDialog = dialog;
		setTitle( resources.getString( "createNewAccount" ) );
	}

	/**
	 * Collects the required registration fields from the server
	**/
	public void getRegistrationInfo()
	{
		Thread thread = new Thread( new GetRegistrationThread() );
		thread.start();
	}

	/**
	 * Sends the registration information to the server
	*/
	public void register()
	{
		Thread thread = new Thread( new RegistrationThread() );
		thread.start();
	}

	/**
	 * Closes the NewAccountDialog
	*/
	public void closeHandler()
	{
		DialogTracker.removeDialog( this );
	}

	/**
	 * Sends the registration information to the server
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class RegistrationThread implements Runnable
	{
		public void run()
		{
			final WaitDialog wait = new WaitDialog( "Please Wait...", "Submitting registration information..." );
			wait.show();
			thisPointer.setVisible( false );
			String errorMessage = null;

			String username = null;
			String password = null;

			Hashtable map = new Hashtable();
			for( int i = 0; i < fieldListNames.size(); i++ )
			{
				String name = (String)fieldListNames.get( i );
				JTextField field = (JTextField)fieldListFields.get( i );

				if( name.equals( "password" ) ) password = field.getText();
				else if( name.equals( "username" ) ) username = field.getText();
				else map.put( name, field.getText() );
			}

			try {
				if( !connection.isConnected() )
				{
					if( !ssl )
					{
						connection = new XMPPConnection( server, port );
					}
					else {
						connection = new SSLXMPPConnection( server, port );
					}
				}

				manager.createAccount( username, password, map );
			}
			catch( XMPPException e )
			{
				errorMessage = e.getMessage();
			}
			catch( IllegalStateException e )
			{
				errorMessage = e.getMessage();
			}

			connection.close();

			final String tempMessage = errorMessage;
			final String tempUsername = username;
			final String tempPassword = password;

			/**
			 * displays an error if there is one or close the registration dialog
			 * if the registration was successful
			 * @author Adam Olsen
			 * @version 1.0
			*/
			SwingUtilities.invokeLater( new Runnable()
			{
				public void run()
				{
					wait.dispose();
					if( tempMessage == null )
					{
						Standard.noticeMessage( null, resources.getString( "createNewAccount" ),
							resources.getString( "accountHasBeenCreated" ) );
						profDialog.setUsername( tempUsername );
						profDialog.setPassword( tempPassword );

						DialogTracker.removeDialog( thisPointer );
					}
					else {
						Standard.warningMessage( BuddyList.getInstance(), resources.getString( "error" ), tempMessage );
						setVisible( true );
					}
				}
			} );
		}
	}

	/**
	 * Thread to get the required fields from the server.
	 * Also builds the dynamic registration form
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class GetRegistrationThread implements Runnable
	{
		private String errorMessage;

		/**
		 * Called by the enclosing thread
		**/
		public void run()
		{
			final WaitDialog wait = new WaitDialog( "Please Wait...", "Collecting registration form..." );
			wait.show();

			try {
				if( !ssl )
				{
					connection = new XMPPConnection( server, port );
				}
				else {
					connection = new SSLXMPPConnection( server, port );
				}
			}
			catch( XMPPException e )
			{
				errorMessage = e.getMessage();
			}

			if( errorMessage == null )
			{
				manager = connection.getAccountManager();

				instructions.setText( "<html><table width='300' border='0'><tr><td align='center'> " +
								manager.getAccountInstructions() + "</td></tr></table></html>" );
				Iterator iterator = manager.getAccountAttributes();

				if( username != null )
				{
					((JTextField)fieldListFields.get( 0 ) ).setText( username );
				}

				if( password != null )
				{
					((JTextField)fieldListFields.get( 1 ) ).setText( password );
				}

				while( iterator.hasNext() )
				{
					String key = (String)iterator.next();
					createInputBox( key );
				}
			}

			/**
			 * Displays the error if there is one, or displays the new RegistrationForm
			 * @author Adam Olsen
			 * @version 1.0
			*/
			SwingUtilities.invokeLater( new Runnable()
			{
				public void run()
				{
					wait.dispose();
					if( errorMessage != null )
					{
						Standard.warningMessage( null, resources.getString( "registration" ), errorMessage );
						DialogTracker.removeDialog( thisPointer );
					}
					else {
						pack();

						setLocationRelativeTo( null );
						setVisible( true );
					}
				}
			} );
		}
	}
}
