/*
 *  Copyright (C) 2003 Adam Olsen
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package com.valhalla.jbother;

import com.valhalla.gui.*;
import java.util.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import java.io.*;
import com.valhalla.jbother.*;
import com.valhalla.settings.*;
import com.valhalla.misc.*;

/**
 * Allows a user to edit a profile
 *
 * @author     synic
 * @created    November 30, 2004
 */
public class ProfileEditorDialog extends JDialog
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );

	private JPanel main;
	private JTabbedPane pane = new JTabbedPane();
	private JTextField nameField = new JTextField();
	private JTextField usernameField = new JTextField( 20 );
	private JTextField serverField = new JTextField( 25 );
	private JTextField resourceField = new JTextField( 20 );
	private JTextField portField = new JTextField( 5 );
	private JPasswordField passwordField = new JPasswordField( 20 );
	private JCheckBox sslBox = new JCheckBox();
	private SettingsProperties settings = new SettingsProperties();
	private JCheckBox autoLoginBox = new JCheckBox();
	private JCheckBox defaultBox = new JCheckBox();
	private JCheckBox reconnectBox = new JCheckBox();
	private JButton createButton = new JButton( resources.getString( "createAccountButton" ) );
	private JButton saveButton = new JButton( resources.getString( "saveButton" ) );
	private JButton cancelButton = new JButton( resources.getString( "cancelButton" ) );
	private ProfileEditorDialog thisPointer = this;
	private File profDir = new File( JBother.settingsDir, "profiles" );
	private JPanel innerPanel = new JPanel();
	private String origProf = null;
	private ProfileManager dialog = null;
	private boolean exitOnClose = false;
	private boolean isCurrentProfile = false;

	/**
	 * Contructs the ProfileEditorDialog
	 *
	 * @param  dialog   the ProfileManager dialog that's calling this editor, or <tt>null</tt> if nothing is calling it
	 * @param  profile  the profile to edit, or <tt>null</tt> if it's a new profile
	 */
	public ProfileEditorDialog( ProfileManager dialog, String profile )
	{
		super( dialog, "Profile Editor", true );
		this.dialog = dialog;

		origProf = profile;

		main = (JPanel)getContentPane();
		main.setLayout( new BorderLayout() );
		main.setBorder( BorderFactory.createTitledBorder( resources.getString( "profileEditor" ) ) );
		main.add( pane, BorderLayout.CENTER );
		pane.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

		JPanel topPanel = new JPanel();
		topPanel.setLayout( new BoxLayout( topPanel, BoxLayout.Y_AXIS ) );

		JPanel namePanel = new JPanel();
		namePanel.setBorder( BorderFactory.createEmptyBorder( 2, 5, 2, 5 ) );
		namePanel.setLayout( new BoxLayout( namePanel, BoxLayout.X_AXIS ) );
		namePanel.add( new JLabel( resources.getString( "profileName" ) + ": " ) );
		namePanel.add( nameField );

		topPanel.add( namePanel );

		JPanel defaultPanel = new JPanel();
		defaultPanel.setBorder( BorderFactory.createEmptyBorder( 0, 5, 0, 5 ) );
		defaultPanel.setLayout( new BoxLayout( defaultPanel, BoxLayout.X_AXIS ) );
		defaultPanel.add( new JLabel( resources.getString( "setAsDefault" ) + ": " ) );
		defaultPanel.add( defaultBox );
		defaultPanel.add( Box.createHorizontalGlue() );

		topPanel.add( defaultPanel );

		main.add( topPanel, BorderLayout.NORTH );

		createAccountPanel();
		createOptionsPanel();

		JPanel buttonPanel = new JPanel();
		buttonPanel.setBorder( BorderFactory.createEmptyBorder( 2, 5, 2, 5 ) );
		buttonPanel.setLayout( new BoxLayout( buttonPanel, BoxLayout.X_AXIS ) );
		buttonPanel.add( Box.createHorizontalGlue() );
		buttonPanel.add( cancelButton );
		buttonPanel.add( saveButton );

		main.add( buttonPanel, BorderLayout.SOUTH );

		addListeners();
		loadProfile( profile );

		pack();
		setLocationRelativeTo( null );
		addWindowListener(
			new WindowAdapter()
			{
				public void windowClosing( WindowEvent e )
				{
					cancelHandler();
				}
			} );
	}

	/**
	 * Sets whether or not this dialog should exit the application when it's cancel button has been pressed
	 *
	 * @param  e  true to close the app
	 */
	protected void setExitOnClose( boolean e )
	{
		this.exitOnClose = true;
	}

	public void setIsCurrentProfile( boolean i )
	{
		this.isCurrentProfile = i;
	}

	/**
	 * @return the defaultBox
	 */
	protected JCheckBox getDefaultBox()
	{
		return defaultBox;
	}

	/**
	 * Adds the event listeners to the buttons
	 */
	private void addListeners()
	{
		PEDialogListener listener = new PEDialogListener();
		createButton.addActionListener( listener );
		saveButton.addActionListener( listener );
		cancelButton.addActionListener( listener );
	}

	/**
	 *  Handles events
	 *
	 * @author     synic
	 * @created    November 30, 2004
	 */
	class PEDialogListener implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			if( e.getSource() == createButton )
			{
				createAccountHandler();
			}
			else if( e.getSource() == saveButton )
			{
				saveHandler();
			}
			else if( e.getSource() == cancelButton )
			{
				cancelHandler();
			}

		}
	}

	/**
	 *  Cancels the dialog, and quits if exitOnClose is set to true
	 */
	private void cancelHandler()
	{
		dispose();
		if( exitOnClose )
		{
			System.exit( 0 );
		}
	}

	/**
	 *  Saves the currently opened profile
	 */
	private void saveHandler()
	{
		try
		{
			Standard.setBundle( resources );
			Standard.assure( nameField.getText(), "Profile Name" );
			Standard.assure( usernameField.getText(), "Username" );
			Standard.assure( new String( passwordField.getPassword() ), "Password" );
			Standard.assure( resourceField.getText(), "Resource" );
			Standard.assure( serverField.getText(), "Server" );
		}
		catch( Exception e )
		{
			com.valhalla.Logger.logException( e );
			return;
		}

		settings.setProperty( "username", usernameField.getText() );
		settings.setProperty( "password", new String( passwordField.getPassword() ) );
		settings.setProperty( "resource", resourceField.getText() );
		settings.setProperty( "defaultServer", serverField.getText() );
		settings.setProperty( "port", portField.getText() );
		settings.setBoolean( "useSSL", sslBox.isSelected() );
		settings.setBoolean( "autoLogin", autoLoginBox.isSelected() );
		settings.setBoolean( "reconnectOnDisconnect", reconnectBox.isSelected() );

		String profile = nameField.getText();
		File profileDir = new File( profDir, profile );

		if( origProf == null )
		{
			// check to see if the profile already exists
			if( profileDir.exists() )
			{
				Standard.warningMessage( this, resources.getString( "profileEditor" ), resources.getString( "profileExists" ) );
				return;
			}

			profileDir.mkdirs();
		}
		else if( !profile.equals( origProf ) )
		{
			File origProfDir = new File( profDir, origProf );
			origProfDir.renameTo( profileDir );
		}

		try
		{
			settings.saveSettings( profDir.getPath() + File.separatorChar + profile + File.separatorChar + "settings.properties",
					"JBother Settings File" );
		}
		catch( IOException ex )
		{
			Standard.warningMessage( this, resources.getString( "profileEditor" ), resources.getString( "errorSavingSettings" ) );
			return;
		}

		if( defaultBox.isSelected() )
		{
			ProfileManager.setDefaultProfile( profile );
		}

		if( dialog != null )
		{
			dialog.loadProfileList();
		}

		if( isCurrentProfile )
		{
			ProfileManager.setCurrentProfile( profile );
			Settings.getInstance().loadSettings( profDir.getPath() + File.separatorChar + profile, "settings.properties" );
		}

		if( exitOnClose )
		{
			ProfileManager.loadProfile( nameField.getText() );
		}

		dispose();
	}

	/**
	 * Calls the NewAccoutDialog to create a new account
	 */
	private void createAccountHandler()
	{
		if( serverField.getText().equals( "" ) )
		{
			Standard.warningMessage( this, "createAccount", resources.getString( "enterNewAccountServer" ) );
			return;
		}

		int port = -1;
		boolean ssl = sslBox.isSelected();

		try
		{
			port = Integer.parseInt( portField.getText() );
		}
		catch( NumberFormatException e )
		{
		}

		if( port == -1 )
		{
			if( ssl )
			{
				port = 5223;
			}
			else
			{
				port = 5222;
			}
		}

		NewAccountDialog dialog = new NewAccountDialog( this, serverField.getText(), usernameField.getText(),
				new String( passwordField.getPassword() ), port, ssl );
		dialog.getRegistrationInfo();
	}

	/**
	 * Sets the username
	 *
	 * @param  username  The new username value
	 */
	public void setUsername( String username )
	{
		usernameField.setText( username );
	}

	/**
	 *  Sets the password
	 *
	 * @param  password  The new password value
	 */
	public void setPassword( String password )
	{
		passwordField.setText( password );
	}

	/**
	 * Creates the Account Panel
	 */
	private void createAccountPanel()
	{
		AbstractOptionPanel panel = new AbstractOptionPanel();
		panel.addComponent( resources.getString( "username" ), usernameField );
		passwordField.setFont( usernameField.getFont() );
		panel.addComponent( resources.getString( "password" ), passwordField );
		panel.addComponent( resources.getString( "resource" ), resourceField );
		panel.addComponent( resources.getString( "server" ), serverField );
		panel.addComponent( createButton, 1, GridBagConstraints.EAST );
		panel.end();

		pane.add( resources.getString( "account" ), panel );
	}

	/**
	 *  Creates the options panel
	 */
	private void createOptionsPanel()
	{
		AbstractOptionPanel panel = new AbstractOptionPanel();
		panel.addComponent( resources.getString( "useSsl" ), sslBox );

		JPanel portPanel = new JPanel( new BorderLayout() );
		portPanel.add( portField, BorderLayout.WEST );
		portPanel.add( new JLabel( " " + resources.getString( "leaveBlankForDefault" ) ), BorderLayout.CENTER );
		panel.addComponent( resources.getString( "logInAutomatically" ), autoLoginBox );
		panel.addComponent( resources.getString( "reconnectOnDisconnect" ), reconnectBox );
		panel.addComponent( resources.getString( "connectPort" ), portPanel );
		panel.end();

		pane.add( resources.getString( "options" ), panel );
	}

	/**
	 * Loads a certain profile
	 *
	 * @param  profile  the profile to load, or null to create a new one
	 */
	private void loadProfile( String profile )
	{
		if( profile != null )
		{
			nameField.setText( profile );

			try
			{
				settings.loadSettings( profDir.getPath() + File.separatorChar + profile + File.separatorChar + "settings.properties" );
			}
			catch( Exception ex )
			{
				return;
			}
		}
		else
		{
			// copy the default file in to place
			InputStream stream = getClass().getClassLoader().getResourceAsStream( "defaultsettings.properties" );
			try
			{
				settings.load( stream );
			}
			catch( Exception ex )
			{
				return;
			}
		}

		usernameField.setText( settings.getProperty( "username" ) );
		passwordField.setText( settings.getProperty( "password" ) );
		resourceField.setText( settings.getProperty( "resource" ) );
		serverField.setText( settings.getProperty( "defaultServer" ) );
		portField.setText( settings.getProperty( "port" ) );

		sslBox.setSelected( settings.getBoolean( "useSSL" ) );
		autoLoginBox.setSelected( settings.getBoolean( "autoLogin" ) );
		reconnectBox.setSelected( settings.getBoolean( "reconnectOnDisconnect" ) );

		String defaultProf = ProfileManager.getDefaultProfile();
		com.valhalla.Logger.debug( defaultProf );

		if( defaultProf != null && defaultProf.equals( profile ) )
		{
			defaultBox.setSelected( true );
		}
	}
}

