/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import org.jivesoftware.smackx.*;
import org.jivesoftware.smackx.packet.*;
import org.jivesoftware.smack.*;

import java.util.*;
import com.valhalla.jbother.*;
import com.valhalla.gui.*;

/**
 * For browsing Jabber services
 *
 * Displays a dialog for browsing services on Jabber
 * entities such as servers according to JEP-0030
 *
 * @author Adam olsen
 * @version 1.0
**/
public class ServiceDiscoveryDialog extends JDialog
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private String host;
	private JButton closeButton = new JButton( resources.getString( "closeButton" ) );
	private JButton searchButton = new JButton( resources.getString( "search" ) );
	private ServiceTableModel tableModel = new ServiceTableModel( this );
	private JTable table = new JTable( tableModel );
	private JLabel title = new JLabel( resources.getString( "serviceDiscoveryManager" ) );
	private JTextField serverField = new JTextField();
	private JLabel status = new JLabel( resources.getString( "status" ) + ": " );
	private JPanel bottomPanel = new JPanel();
	private JPanel middlePanel = new JPanel( new BorderLayout( 5, 5 ) );
	private JPanel topPanel = new JPanel();
	private JLabel hostLabel = new JLabel( resources.getString( "host" ) + ": " );
	private ServiceDiscoveryThread currentThread = null;
	private ServiceDiscoveryDialog thisPointer = this;
	private TablePopupMenu popupMenu = new TablePopupMenu( this, table );

	/**
	 * Creates a ServiceDiscoveryDialog with parent as it's parent
	 * @param parent the JFrame that owns this dialog
	**/
	public ServiceDiscoveryDialog( JFrame parent )
	{
		super( parent );
		setTitle( resources.getString( "serviceDiscovery" ) );

		initComponents();
	}

	/**
	 * Sets up the Dialog layout
	 **/
	private void initComponents()
	{
		tableModel.setTable( table );
		JPanel panel = (JPanel)getContentPane();

		panel.setLayout( new BorderLayout( 5, 5 ) );
		panel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

		JScrollPane scrollPane = new JScrollPane( table );
		scrollPane.getViewport().setBackground( Color.WHITE );

		topPanel.setLayout( new BorderLayout( 5, 5 ) );
		topPanel.add( hostLabel, BorderLayout.WEST );
		topPanel.add( serverField, BorderLayout.CENTER );

		middlePanel.add( topPanel, BorderLayout.NORTH );

		middlePanel.add( scrollPane, BorderLayout.CENTER );

		panel.add( title, BorderLayout.NORTH );
		table.setBorder( BorderFactory.createEtchedBorder() );
		panel.add( middlePanel, BorderLayout.CENTER );

		bottomPanel.setLayout( new BoxLayout( bottomPanel, BoxLayout.X_AXIS ) );
		bottomPanel.add( status );

		bottomPanel.add( Box.createHorizontalGlue() );
		bottomPanel.add( searchButton );
		bottomPanel.add( closeButton );

		panel.add( bottomPanel, BorderLayout.SOUTH );
		pack();
		setSize( new Dimension( 600, 300 ) );

		if( BuddyList.getInstance().checkConnection() ) serverField.setText( BuddyList.getInstance().getConnection().getHost() );

		ActionListener listener = new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				runServiceDiscovery( null );
			}
		};

		serverField.addActionListener( listener );
		searchButton.addActionListener( listener );

		Standard.cascadePlacement( this );

		closeButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e ) { dispose(); }
		} );

		table.addMouseListener( new PopupMouseListener() );

		runServiceDiscovery( null );
	}

	/**
	 * Listens for mouse events
	 * @author Adam Olsen
	 * @version 1.0
	 **/
	class PopupMouseListener extends MouseAdapter
	{
		public void mousePressed( MouseEvent e ) { checkPop( e ); }
		public void mouseReleased( MouseEvent e ) { checkPop( e ); }
		public void mouseClicked( MouseEvent e )
		{
			if( e.getClickCount() >= 2 )
			{
				int row = table.getSelectedRow();
				if( row < 0 ) return;

				String id = (String)tableModel.getValueAt( row, 1 );
				runServiceDiscovery( id );
			}
			else checkPop( e );
		}

		public void checkPop( MouseEvent e )
		{
			if( e.isPopupTrigger() )
			{
				popupMenu.popup( e );
			}
		}
	}

	/**
	 * Starts the service discovery thread on the specified server
	 * @param server the server to run discovery on
	 **/
	protected void runServiceDiscovery( String server )
	{
		if( !BuddyList.getInstance().checkConnection() )
		{
			BuddyList.getInstance().connectionError();
			return;
		}

		if( server != null ) serverField.setText( server );
		if( serverField.getText().equals( "" ) )
		{
			Toolkit.getDefaultToolkit().beep();
			return;
		}

		status.setText( resources.getString( "status" ) + ": " + resources.getString( "collecting" ) + " ..." );
		tableModel.clear();

		if( currentThread != null ) currentThread.abortDiscovery();
		currentThread = new ServiceDiscoveryThread();
		new Thread( currentThread ).start();
	}

	/**
	 * The thread that actually collects disco information about the server
	 * @author Adam Olsen
	 * @version 1.0
	 **/
	class ServiceDiscoveryThread implements Runnable
	{
		private boolean stopped = false;
		private ArrayList discoItems = new ArrayList();
		public void run()
		{
			if( !BuddyList.getInstance().checkConnection() )
			{
				BuddyList.getInstance().connectionError();
				return;
			}

			ServiceDiscoveryManager manager = new ServiceDiscoveryManager( BuddyList.getInstance().getConnection() );

			// get the discover items for the server
			try {
				DiscoverItems items = manager.discoverItems( serverField.getText() );
				Iterator i = items.getItems();

				String top[] = new String[] { serverField.getText(), serverField.getText(), "", "", "" };
				discoItems.add( top );
				tableModel.addItem( top );

				while( i.hasNext() )
				{
					DiscoverItems.Item item = (DiscoverItems.Item)i.next();
					if( stopped ) return;

					final String[] entry = new String[] { item.getName(), item.getEntityID(), "", item.getNode(), item.getAction() };
					discoItems.add( entry );

					SwingUtilities.invokeLater( new Runnable()
					{
						public void run()
						{
							tableModel.addItem( entry );
						}
					} );
				}

				for( int icount = 0; icount < discoItems.size(); icount++ )
				{
					String[] entry = (String[])discoItems.get( icount );
					String id = entry[1];

					status.setText( resources.getString( "status" ) + ": " +
						resources.getString( "gettingFeatures" ) + " (" + id + ") ..." );

					// get the discover info about each item
					DiscoverInfo info = null;

					try {
						info = manager.discoverInfo( id );
					}
					catch( XMPPException e ) { }

					// if the service discovery has been aborted, bail out
					if( stopped ) return;
					tableModel.setDisco( icount, info );

					if( info != null )
					{
						final DiscoverInfo tempInfo = info;
						final int index = icount;
						SwingUtilities.invokeLater( new Runnable()
						{
							public void run()
							{
								Iterator identities = tempInfo.getIdentities();
								while( identities.hasNext() )
								{
									DiscoverInfo.Identity identity = (DiscoverInfo.Identity)identities.next();
									if( stopped ) return;
									// set the table information
									tableModel.setItemInfo( index, identity.getName(), identity.getCategory() );
								}
							}
						} );
					}
				}

			}
			catch( XMPPException e )
			{
				status.setText( resources.getString( "error" ) + ": " + e.getMessage() );
				return;
			}

			SwingUtilities.invokeLater( new Runnable()
			{
				public void run()
				{
					status.setText( resources.getString( "status" ) + ": " + resources.getString( "completed" ) + "." );
				}
			} );
		}

		/**
		 * Aborts the service discovery
		 **/
		public void abortDiscovery() { this.stopped = true; }
	}
}

/**
 * The popup menu for each of the disco items
 * @author Adam Olsen
 * @version 1.0
 **/
class TablePopupMenu extends JPopupMenu
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JTable table;
	private ServiceTableModel model;
	private ServiceDiscoveryDialog dialog;
	private JMenuItem browseItem = new JMenuItem( resources.getString( "browse" ) );
	private JMenuItem registerItem = new JMenuItem( resources.getString( "register" ) );
	private JMenuItem addItem = new JMenuItem( resources.getString( "addToRoster" ) );

	/**
	 * Default constructor
	 * @param dialog The ServiceDiscoveryDialog to connect this popup menu to
	 * @param table the table to connect this menu to
	 **/
	public TablePopupMenu( ServiceDiscoveryDialog dialog, JTable table )
	{
		this.dialog = dialog; this.table = table;
		model = (ServiceTableModel)table.getModel();
		final TablePopupMenu thisPointer = this;

		add( addItem );
		add( browseItem );
		add( registerItem );

		// show the add buddy dialog
		addItem.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				AddBuddyDialog dialog = new AddBuddyDialog();
				String id = getId();
				dialog.setBuddyId( id );
				dialog.setVisible( true );
			}
		} );

		// the browse item runs service discovery on that item
		browseItem.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				String id = getId();
				thisPointer.dialog.runServiceDiscovery( id );
			}
		} );

		// the register item forms a RegistrationForm for that item
		registerItem.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				String id = getId();
				new RegistrationForm( id ).getRegistrationInfo();
			}
		} );
	}

	/**
	 * Returns the ID of the selected row
	 * @return the id of the row
	 **/
	private String getId()
	{
		int row = this.table.getSelectedRow();
		if( row < 0 ) return "";

		String id = (String)model.getValueAt( row, 1 );
		return id;
	}

	/**
	 * Shows the popup menu
	 * @param e the mouse event
	 **/
	public void popup( MouseEvent e )
	{
		int selectedRow = table.rowAtPoint( e.getPoint() );
		if( selectedRow < 0 ) return;
		table.setRowSelectionInterval( selectedRow, selectedRow );

		/*String features = model.getFeatures( selectedRow );
		if( features.indexOf( "r" ) > -1 ) registerItem.setEnabled( true );
		else registerItem.setEnabled( false );*/

		show( table, e.getX(), e.getY() );
	}
}

/**
 * The table model for the ServiceDiscoveryDialog table
 * @author Adam Olsen
 * @version 1.0
**/
class ServiceTableModel extends AbstractTableModel
{
	private ServiceDiscoveryDialog dialog;
	private JTable table;
	private ArrayList items = new ArrayList();
	private String[] columns = new String[] { "Name", "JID", "Category", "Node", "Action" };
	private ArrayList infos = new ArrayList();

	/**
	 * Sets the table value for this model
	 * @param table the table that this model represents
	**/
	public void setTable( JTable table )
	{
		this.table = table;

		TableColumn column = null;

		// set the default column widths
		for( int i = 0; i < getColumnCount(); i++)
		{
			column = table.getColumnModel().getColumn( i );
			if( i < 2 ) column.setPreferredWidth( 150 ); // sport column is bigger
			else column.setPreferredWidth( 50 );
		}
	}

	/**
	 * Default Constructor
	 * @param dialog the ServiceDiscoveryDialog that contains this table
	*/
	public ServiceTableModel( ServiceDiscoveryDialog dialog ) { this.dialog = dialog; }

	/**
	 * gets the number of columns in this table
	 * @return the number of columns
	*/
	public int getColumnCount() { return columns.length; }

	/**
	 * Gets the number of rows in the table
	 * @return the number of rows in the table
	*/
	public int getRowCount() { return items.size(); }

	/**
	 * Returns the name of a specific column
	 * @param column the column who's name is wanted
	 * @return the name of the column
	*/
	public String getColumnName( int column ) { return columns[column]; }

	/**
	 * Get the Object for a specific coordinate in the table
	 * @param row the row of the item
	 * @param column the column of the item
	 * @return the Object at the specified coordinates
	*/
	public Object getValueAt( int row, int column )
	{
		synchronized( items )
		{
			String[] item = (String[])items.get( row );
			return item[column];
		}
	}

	/**
	 * gets the features of a specific item (row)
	 * @param row 	the index of the row you want information for
	 * @return	a string containing all of the features the row supports
	*/
	public String getFeatures( int row )
	{
		synchronized( infos )
		{
			String info = "";

			try {
				info = (String)infos.get( row );
			}
			catch( Exception e ) { }

			return info;
		}
	}

	/**
	 * Sets the disco information once it's found
	 * @param index	the index of the row you want to set the information about
	 * @param disco	the information about the row
	*/
	public void setDisco( int index, DiscoverInfo disco )
	{
		synchronized( infos )
		{
			String info = "";
			if( disco != null && disco.containsFeature( "jabber:iq:register" ) ) info += "r";
			infos.add( index, info );
		}
	}

	/**
	 * Adds a row to the table
	 * @param item	the array containing the item to add
	*/
	public void addItem( String[] item )
	{
		synchronized( items )
		{
			items.add( item );
			fireTableRowsInserted( items.size(), items.size() );
		}
	}

	/**
	 * Sets information about a specific row
	 * @param row the row to set
	 * @param name the new name
	 * @param category the category of the row
	*/
	public void setItemInfo( int row, String name, String category )
	{
		synchronized( items )
		{
			String[] item = (String[])items.get( row );
			item[0] = name;
			item[2] = category;

			fireTableRowsUpdated( row, row );
		}
	}

	/**
	 * Clears the table
	*/
	public void clear()
	{
		items.clear();
		infos.clear();
		table.tableChanged( new TableModelEvent( this ) );
	}
}
