/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.jivesoftware.smack.packet.Presence;
import com.valhalla.settings.Settings;
import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.groupchat.*;
import com.valhalla.jbother.jabber.*;

/**
 * Contains all of the groupchat windows in tabs
 *
 * @author Adam Olsen
 * @version 1.0
 **/
public class TabFrame extends JFrame
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JPanel container = new JPanel( new BorderLayout() );
	private JTabbedPane tabPane = new JTabbedPane();
	private JMenuBar menuBar = new JMenuBar();
	private JMenu optionMenu = new JMenu( resources.getString( "options" ) );
	private JMenuItem	newItem = new JMenuItem( resources.getString( "joinRoom" ) ),
						leaveItem = new JMenuItem( resources.getString( "leaveAll" ) );
	private Hashtable queueCounts = new Hashtable();
	private GCTabHandler tabListener = new GCTabHandler();

	/**
	 * Constructor sets the frame up and adds a listener to the JTabPane so that if
	 * a tab is changed the title of this frame reflects the topic and the name of the room
	 * in the tab
	 **/
	public TabFrame()
	{
		super( "Chat Window" );
		setTitle( resources.getString( "chatWindowTitle" ) );

		ImageIcon icon = StatusIconCache.getStatusIcon(org.jivesoftware.smack.packet.Presence.Mode.AVAILABLE);
		if (icon != null ) setIconImage( icon.getImage() );

		setContentPane( container );
		tabPane.setTabPlacement( JTabbedPane.BOTTOM ) ;
		container.add( tabPane, BorderLayout.CENTER );

		//add the tab switch listener so the title of the frame reflects the current tab
		//contents
		tabPane.addChangeListener( new ChangeListener()
		{
			public void stateChanged( ChangeEvent e )
			{
				if( tabPane.getSelectedComponent() != null )
				{
					TabFramePanel panel = (TabFramePanel)tabPane.getSelectedComponent();
					setTitle( panel.getWindowTitle() );
					panel.focusYourself();
					tabPane.setTitleAt( tabPane.indexOfComponent( (Component)panel ), panel.getPanelName() );
					queueCounts.remove( panel );
				}
			}
		} );

		optionMenu.add( newItem );
		optionMenu.add( leaveItem );

		addListeners();

		menuBar.add( optionMenu );

		//if they press the close button, we wanna handle leaving of the rooms
		//gracefully
		addWindowListener( new WindowAdapter()
		{
			public void windowClosing( WindowEvent e ) { closeHandler(); }
		} );

		setPreferredLocation();
		pack();

		String stringWidth = Settings.getInstance().getProperty( "chatFrameWidth" );
		String stringHeight = Settings.getInstance().getProperty( "chatFrameHeight" );
		if( stringWidth == null ) stringWidth = "635";
		if( stringHeight == null ) stringHeight = "450";

		setSize( new Dimension( Integer.parseInt( stringWidth ), Integer.parseInt( stringHeight ) ) );
		DialogTracker.addDialog( this, true, false );

		addComponentListener( new ComponentAdapter()
		{
			public void componentResized( ComponentEvent e )
			{
				saveStates();
			}
		} );
	}

	/**
	 * Marks a tab for a TabFramePanel if it's not already selected
	 * @param panel the panel to mark
	*/
	public void markTab( TabFramePanel panel )
	{
		if( tabPane.getSelectedComponent() != panel )
		{
			Integer i = (Integer)queueCounts.get( panel );
			if( i == null ) i = new Integer( 1 );

			int index = tabPane.indexOfComponent( (Component)panel );
			tabPane.setTitleAt( index, panel.getPanelName() + " (" + i.intValue() + ")" );

			queueCounts.put( panel, new Integer( i.intValue() + 1 ) );
		}
	}

	/**
	 * Saves the size of the chat frame
	*/
	public void saveStates()
	{
		Dimension size = getSize();
		Integer width = new Integer( (int)size.getWidth() );
		Integer height = new Integer( (int)size.getHeight() );
		Settings.getInstance().setProperty( "chatFrameWidth", width.toString() );
		Settings.getInstance().setProperty( "chatFrameHeight", height.toString() );
	}

	/**
	 * Loads the saved settings from any previous settings
	 */
	private void setPreferredLocation()
	{
		//load the settings from the settings file
		String xString = Settings.getInstance().getProperty( "chatFrameX" );
		String yString = Settings.getInstance().getProperty( "chatFrameY" );

		if( yString == null ) yString = "100";
		if( xString == null ) xString = "100";

		double x = Double.parseDouble( xString );
		double y = Double.parseDouble( yString );

		if( x < 0.0 ) x = 100.0;
		if( y < 0.0 ) y = 100.0;

		setLocation( (int)x, (int)y );
	}

	/**
	 * Adds the various event listeners
	 * @author Adam Olsen
	 * @version 1.0
	*/
	private void addListeners()
	{
		MenuItemListener listener = new MenuItemListener();
		newItem.addActionListener( listener );
		leaveItem.addActionListener( listener );
		tabPane.addMouseListener( new MouseAdapter()
		{
			public void mouseReleased( MouseEvent e )
			{
				TabFramePanel panel = (TabFramePanel)tabPane.getSelectedComponent();
				panel.focusYourself();
			}
		} );
	}

	/**
	 * Listens for a menu item to be clicked
	 * @author Adam Olsen
	 * @version 1.0
	*/
	private class MenuItemListener implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			if( e.getSource() == newItem ) new GroupChatBookmarks().setVisible( true );
			if( e.getSource() == leaveItem ) closeHandler();
		}
	}

	/**
	 * Updates the font in all the chat conversationareas
	 * @param font the font to update to
	*/
	public void updateStyles( Font font )
	{
		for( int i = 0; i < tabPane.getTabCount(); i++ )
		{
			TabFramePanel panel = (TabFramePanel)tabPane.getComponentAt( i );
			panel.updateStyle( font );
		}
	}


	/**
	 * Set the status in all the rooms
	 * @param mode the presence mode
	 * @param status the status string
	 **/
	public void setStatus( Presence.Mode mode, String status )
	{
		for( int i = 0; i < tabPane.getTabCount(); i++ )
		{
			TabFramePanel panel = (TabFramePanel)tabPane.getComponentAt( i );
			if( panel instanceof ChatRoomPanel )
			{
				ChatRoomPanel window = (ChatRoomPanel)panel;

				//set up a packet to be sent to my user in every groupchat
				Presence presence = new Presence( Presence.Type.AVAILABLE, status, 0, mode );
				presence.setTo( window.getRoomName() + '/' + window.getNickname() );

				if( !BuddyList.getInstance().checkConnection() )
				{
					BuddyList.getInstance().connectionError();
					return;
				}

				BuddyList.getInstance().getConnection().sendPacket( presence );
			}
		}
	}

	/**
	 * This not only closes the window, but it leaves all the rooms like it should
	 **/
	public void closeHandler()
	{
		leaveAll();
	}

	/**
	 * Since there is no way to check to see if a message is from someone in a chat room, we check to see if
	 * the message is coming from the same server as a chatroom we are in.
	 * @param server the server to check
	 **/
	public boolean isRoomOpen( String server )
	{
		for( int i = 0; i < tabPane.getTabCount(); i++ )
		{
			TabFramePanel panel = (TabFramePanel)tabPane.getComponentAt( i );
			if( panel instanceof ChatRoomPanel )
			{
				ChatRoomPanel window = (ChatRoomPanel)panel;
				if( server.toLowerCase().equals( window.getRoomName().toLowerCase() ) )	return true;
			}
		}

		return false;
	}

	/**
	 * If there is a chatroom open in this frame with a server name, this returns the ChatRoomPanel that contains it
	 * @param server the name of the room to get the ChatRoomPanel for
	 * @return the ChatRoomPanel requested, or <tt>null</tt> if it could not be found
	 **/
	public ChatRoomPanel getChatPanel( String server )
	{
		for( int i = 0; i < tabPane.getTabCount(); i++ )
		{
			TabFramePanel panel = (TabFramePanel)tabPane.getComponentAt( i );
			if( panel instanceof ChatRoomPanel )
			{
				ChatRoomPanel window = (ChatRoomPanel)panel;
				if( server.toLowerCase().equals( window.getRoomName().toLowerCase() ) ) return window;
			}
		}

		return null;
	}

	/**
	 * This leaves a chatroom and removes the associated ChatRoomPanel from the TabPane
	 * @param window the room to leave
	 **/
	public void removePanel( TabFramePanel panel )
	{
		tabPane.remove( (Component)panel );
		tabPane.validate();

		if( panel instanceof ChatRoomPanel )
		{
			((ChatRoomPanel)panel).leave();
		}
	}

	/**
	 * Sets the subject of a ChatRoomPanel based on a message that was recieved from the GroupChat server with
	 * &lt;subject&gt; in it
	 * @param window the window to set the subject for
	 **/
	public void setSubject( ChatRoomPanel window )
	{
		if( !( tabPane.getSelectedComponent() instanceof ChatRoomPanel ) ) return;
		if( (ChatRoomPanel)tabPane.getSelectedComponent() == window )
		{
			setTitle( resources.getString( "groupChat" ) + ": " + window.getRoomName() );
			validate();
		}
	}

	/**
	 * @param panel the panel to check
	 * @return true if the tab panel is currently displayed in the tab frame
	*/
	public boolean contains( TabFramePanel panel )
	{
		for( int i = 0; i < tabPane.getTabCount(); i++ )
		{
			TabFramePanel p = (TabFramePanel)tabPane.getComponentAt( i );
			if( p == panel ) return true;
		}

		return false;
	}

	/**
	 * Returns the number of rooms currently open in the frame
	 * @return the number of rooms still open
	 **/
	public int tabsLeft() { return tabPane.getTabCount(); }

	/**
	 * Adds a chat room to the frame
	 * @param window the room to add
	*/
	public void addPanel( TabFramePanel panel )
	{
		tabPane.addTab( panel.getPanelName(), (Component)panel );
		if( panel instanceof ChatRoomPanel )
			tabPane.setSelectedComponent( (Component)panel );
		panel.getInputComponent().addKeyListener( tabListener );
		panel.setListenersAdded( true );
	}

	/**
	 * Switches the current tab in the tab frame
	*/
	public void switchTab()
	{
		int current = tabPane.getSelectedIndex();
		current++;
		if( current >= tabPane.getTabCount() ) current = 0;
		tabPane.setSelectedIndex( current );
	}

	/**
	 * Switches the tab based on CTRL+n keys
	*/
	class GCTabHandler extends KeyAdapter
	{
		public void keyTyped( KeyEvent e )
		{
			// get the ASCII character code for the character typed
			int numPressed = (int)e.getKeyChar();

			// the integer characters start at ASCII table number 49, so we subtract 49
			numPressed -= 49;

			// if the new ASCII value is between 0 and 8, then the
			// key pressed was 1 through 9 - which is what we want
			// also check that the CTRL key was being held down
			if( ( numPressed >= 0 && numPressed <= 8 ) &&
				( e.getModifiers() & KeyEvent.CTRL_MASK ) == Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() )
			{
				e.consume();

				if( tabPane.getTabCount() >= numPressed )
					tabPane.setSelectedIndex( numPressed );
			}
		}
	}

	/**
	 * Leaves all chatrooms (for if they close the window)
	 **/
	public void leaveAll()
	{
		com.valhalla.Logger.debug( "There are " + tabPane.getTabCount() + " rooms" );

		int tabCount = tabPane.getTabCount();
		for( int i = 0; i < tabCount; i++ )
		{
			TabFramePanel panel = (TabFramePanel)tabPane.getComponentAt( 0 );

			if( panel instanceof ChatRoomPanel )
			{
				ChatRoomPanel window = (ChatRoomPanel)panel;
				//if this frame is closed as a result of connection loss and we try to leave
				//the channel, it will not work, so we need to catch it.
				try {
					window.leave();
				}
				catch( IllegalStateException e )
				{
					com.valhalla.Logger.debug( "Caught Illegal State Exception when leaving window: " + window.toString() );
				}

				BuddyList.getInstance().removeTabPanel( panel );
			}
			else {
				((ConversationPanel)panel).checkCloseHandler();
			}
		}

		BuddyList.getInstance().stopTabFrame();
	}

	/**
	 * @return Returns the tabPane.
	 */
	public JTabbedPane getTabPane() {
		return tabPane;
	}
}
