/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.groupchat;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.settings.Settings;

/**
 * Allows the user to create bookmarks for each of their favorite group chat rooms
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class GroupChatBookmarks extends JDialog
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JPanel container = new JPanel();
	private JList bookMarkList;
	private JPanel rightPanel = new JPanel();
	private JPanel leftPanel = new JPanel();
	private JPanel buttonPanel = new JPanel();
	private JPanel inputPanel = new JPanel();
	private JButton			saveButton = new JButton( resources.getString( "saveButton" ) ),
							openButton = new JButton( resources.getString( "openButton" ) ),
							cancelButton = new JButton( resources.getString( "cancelButton" ) );
	private File bookMarksDirectory;
	private JTextField		roomBox = new JTextField( 15 ),
							serverBox = new JTextField( 20 ),
							nickBox = new JTextField( 15 );
	private int row = 0;
	private GridBagLayout grid = new GridBagLayout();
	private GridBagConstraints c = new GridBagConstraints();
	private File fileList[];
	private JPopupMenu deleteMenu = new JPopupMenu();
	private JMenuItem deleteItem = new JMenuItem( resources.getString( "deleteButton" ) );
	private String lastOpenedBookmark;

	/**
	 * Sets up the visual components of the bookmark dialog
	*/
	public GroupChatBookmarks()
	{
		super( BuddyList.getInstance(), "Group Chat Bookmarks" );
		setTitle( resources.getString( "groupChatBookmarksDialogTitle" ) );

		container.setLayout( new BoxLayout( container, BoxLayout.X_AXIS ) );
		container.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

		bookMarkList = new JList();
		bookMarkList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		bookMarkList.setCellRenderer( new ListRenderer() );


		leftPanel.setBackground( Color.WHITE );
		leftPanel.setLayout( new GridLayout( 0, 1 ) );
		setContentPane( container );

		deleteMenu.add( deleteItem );

		bookMarksDirectory = new File( JBother.profileDir + File.separatorChar + "gcbookmarks" );

		loadBookmarks();
		leftPanel.add( new JScrollPane( bookMarkList ) );
		leftPanel.setPreferredSize( new Dimension( 120, 200 ) );

		inputPanel.setBorder( BorderFactory.createTitledBorder( resources.getString( "groupChatBookmarksDialogTitle" ) ) );
		rightPanel.setLayout( new BoxLayout( rightPanel, BoxLayout.Y_AXIS ) );

		inputPanel.setLayout( grid );
		c.anchor = GridBagConstraints.WEST;

		createInputBox( resources.getString( "room" ) + ":", roomBox );
		createInputBox( resources.getString( "server" ) + ":", serverBox );
		createInputBox( resources.getString( "nickname" ) + ":", nickBox );

		container.add( leftPanel );
		container.add( Box.createRigidArea( new Dimension( 5, 0 ) ) );

		//this is the space taker
		JLabel blankLabel = new JLabel( "" );
		c.weighty = 1;
		c.weightx = 1;
		c.gridx = 0;
		c.gridwidth = 3;
		c.gridy++;
		grid.setConstraints( blankLabel, c );
		inputPanel.add( blankLabel );
		buttonPanel.setLayout( new BoxLayout( buttonPanel, BoxLayout.X_AXIS ) );
		buttonPanel.add( Box.createHorizontalGlue() );
		buttonPanel.add( saveButton );
		buttonPanel.add( openButton );
		buttonPanel.add( cancelButton );


		rightPanel.add( inputPanel );
		rightPanel.add( buttonPanel );
		container.add( rightPanel );

		setListeners();

		setSize( 400, 200 );
		pack();
		setLocationRelativeTo( null );
		DialogTracker.addDialog( this, true, true );
	}

	/**
	 * Adds event listeners to components in the bookmark window
	*/
	private void setListeners()
	{
		setDefaultCloseOperation( WindowConstants.DISPOSE_ON_CLOSE );
		cancelButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e ) { dispose(); }
		} );

		saveButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e ) { saveBookmark(); }
		} );

		openButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e ) { openHandler(); }
		} );

		bookMarkList.addMouseListener( new RightClickListener() );
		deleteItem.addActionListener( new DeleteListener() );
	}

	/**
	 * Writes a bookmark to the bookmarks file on disk
	*/
	private void saveBookmark()
	{
		if( !checkData() ) return;

		if( !bookMarksDirectory.isDirectory() && !bookMarksDirectory.mkdir() )
		{
			com.valhalla.Logger.debug( "Could not create bookmarks directory." );
			return;
		}

		String defaultString = lastOpenedBookmark;
		if( lastOpenedBookmark == null || lastOpenedBookmark.equals( "" ) ) defaultString = roomBox.getText();

		String result = (String)JOptionPane.showInputDialog( null,
			resources.getString( "enterBookmarkName" ),
			resources.getString( "saveBookmark" ), JOptionPane.QUESTION_MESSAGE, null, null, defaultString );

		if( result == null || result.equals( "" ) ) return;

		try {
			FileWriter fw = new FileWriter( new File( bookMarksDirectory, result + ".gcb" ) );
			PrintWriter out = new PrintWriter( fw );

			out.println( roomBox.getText() );
			out.println( serverBox.getText() );
			out.println( nickBox.getText() );

			fw.close();
		}
		catch( IOException e ) { Standard.warningMessage( this, resources.getString( "saveBookmark" ),
			resources.getString( "unkownError" ) ); }

		loadBookmarks();
	}

	/**
	 * Makes sure all of the required information to save a bookmark is filled in
	 * @return true if the information is completely filled out
	*/
	private boolean checkData()
	{
		if( roomBox.getText().equals( "" ) ||
			serverBox.getText().equals( "" ) || nickBox.getText().equals( "" ) )
		{
			Standard.warningMessage( this, resources.getString( "enterAllFields" ),
				resources.getString( "groupChatBookmarksDialogTitle" ) );
			return false;
		}

		return true;
	}

	/**
	 * Deletes a bookmark from disk
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class DeleteListener implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			int result = JOptionPane.showConfirmDialog( null,
				resources.getString( "sureDelete" ), resources.getString( "deleteBookmark" ), JOptionPane.YES_NO_OPTION );

			if( result != 0 ) return;

			ListModel model = bookMarkList.getModel();
			File file = (File)model.getElementAt( bookMarkList.getSelectedIndex() );
			file.delete();
			loadBookmarks();
			validate();
		}
	}

	/**
	 * Shows a menu on the bookmark items allowing you to delete one of them
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class RightClickListener extends MouseAdapter
	{
		public void mousePressed( MouseEvent e ) { checkPop( e ); }
		public void mouseReleased( MouseEvent e ) { checkPop( e ); }
		public void mouseClicked( MouseEvent e ) { checkPop( e ); }

		public void checkPop( MouseEvent e )
		{
			if( e.isPopupTrigger() )
			{
				int index = bookMarkList.locationToIndex( new Point( e.getX(), e.getY() ) );
				bookMarkList.setSelectedIndex( index );
				deleteMenu.show( e.getComponent(), e.getX(), e.getY() );
			}
			else {
				loadBookmark();
				if( e.getClickCount() >= 2 ) openHandler();
			}
		}
	}

	/**
	 * Opens a bookmark and fills out the fields with it's information
	*/
	private void loadBookmark()
	{
		ListModel model = bookMarkList.getModel();
		File file = (File)model.getElementAt( bookMarkList.getSelectedIndex() );

		try {
			FileReader fr = new FileReader( file );
			BufferedReader in = new BufferedReader( fr );

			String line = in.readLine();
			if( line != null ) roomBox.setText( line );
			line = in.readLine();
			if( line != null ) serverBox.setText( line );
			line = in.readLine();
			if( line != null ) nickBox.setText( line );

			fr.close();
		}
		catch( IOException e )
		{
			Standard.warningMessage( this, resources.getString( "openBookmark" ),
				resources.getString( "unkownError" ) );
			return;
		}

		lastOpenedBookmark = file.getName().replaceAll( "\\.gcb", "" ).replaceAll( ".*\\" + File.separatorChar, "" );
	}

	/**
	 * Opens a connection to a groupchat room
	*/
	private void openHandler()
	{
		checkData();

		if( BuddyList.getInstance().getTabFrame() != null &&
			BuddyList.getInstance().getTabFrame().isRoomOpen( roomBox.getText() + "@" + serverBox.getText() ) )
		{
			Standard.warningMessage( this, resources.getString( "openBookmark" ), resources.getString( "alreadyInRoom" ) );
			return;
		}

		ChatRoomPanel window = new ChatRoomPanel( roomBox.getText() + "@" + serverBox.getText(), nickBox.getText() );
		window.startChat();
		dispose();
	}

	/**
	 * Loads all the bookmarks and puts them in the list of bookmarks
	*/
	private void loadBookmarks()
	{
		if( !bookMarksDirectory.isDirectory() )
		{
			bookMarkList.setListData( new String[] { } );
			return;
		}

		fileList = bookMarksDirectory.listFiles( new GCFileFilter() );
		if( fileList.length == 0 )
		{
			bookMarkList.setListData( new String[] { } );
			return;
		}

		Arrays.sort( fileList );

		bookMarkList.setListData( fileList );
	}

	/**
	 * Creates a JTextField with a label
	 * @param label the name of the the label
	 * @param box the JTextField
	*/
	private void createInputBox( String label, Container box )
	{
		JLabel labelBox = new JLabel( label + "    " );

		c.gridy = row++;
		c.gridx = 0;
		grid.setConstraints( labelBox, c );
		inputPanel.add( labelBox );

		c.gridx = 1;
		grid.setConstraints( box, c );
		inputPanel.add( box );
	}
}

/**
 * Displays the different group chat bookmarks
 * @author Adam Olsen
 * @version 1.0
*/
class ListRenderer extends JLabel implements ListCellRenderer
{
	public ListRenderer()
	{
		setOpaque( true );
	}

	public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSelected, boolean cellHasFocus )
	{
		File file = (File)value;

		setText( file.toString().replaceAll( "\\.gcb", "" ).replaceAll( ".*\\" + File.separatorChar, "" ) );

		setBackground( isSelected ? list.getSelectionBackground() : list.getBackground() );
		setForeground( isSelected ? list.getSelectionForeground() : list.getForeground() );
		list.validate();

		return this;
	}
}

/**
 * Filters out everything but .gcb extensions
 * @author Adam Olsen
 * @version 1.0
*/
class GCFileFilter implements FileFilter
{
	/**
	 * @return true if the file is a directory or a .gcb file
	*/
	public boolean accept( File f )
	{
		if( f.isDirectory() ) return false;

		String extension = Utils.getExtension( f );

		if( extension != null )
		{
			if( extension.equals( Utils.gcb ) )
			{
					return true;
			}
			else {
				return false;
			}
		}

		return false;
	}
}
