/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.groupchat;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import com.valhalla.gui.*;
import com.valhalla.jbother.jabber.BuddyStatus;
import com.valhalla.jbother.menus.*;
import com.valhalla.jbother.*;

/**
 * The JPanel that contains the nickname list
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class GroupChatNickList extends JPanel
{
	private NickListModel nickListModel = new NickListModel();
	private JList nickList = new JList( nickListModel );
	private JScrollPane scrollPane = new JScrollPane( nickList );
	private ChatRoomPanel window;
	private NickListPopupMenu popMenu = new NickListPopupMenu();

	/**
	 * Sets up the panel
	 * @param window the chatroom window that this nicklist is a part of
	*/
	public GroupChatNickList( ChatRoomPanel window )
	{
		super();

		this.window = window;
		setLayout( new GridLayout( 0, 1 ) );

		add( scrollPane );
		setPreferredSize( new Dimension( 100, 400 ) );
		NickListRenderer renderer = new NickListRenderer( window );
		nickList.setCellRenderer( renderer );
		nickList.addMouseListener( new DoubleClickListener() );
	}

	/**
	 * Gets the JList
	 * @return the JList
	*/
	public JList getList() { return nickList; }

	/**
	 * Adds a buddy to the JList (when they sign on)
	 * @param buddy the buddy to add
	*/
	public void addBuddy( String buddy ) { nickListModel.addBuddy( buddy ); }

	/**
	 * Removes a buddy from the JList
	 * @param buddy the buddy to remove
	*/
	public void removeBuddy( String buddy ) { nickListModel.removeBuddy( buddy ); }

	/**
	 * The model that represents the list of buddies in the room
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class NickListModel extends AbstractListModel
	{
		private ArrayList buddies = new ArrayList();
		private Object[] buddyNames = null;

		/**
		 * @return the number of elements in the list
		*/
		public int getSize()
		{
			if( buddyNames == null ) return 0;

			return buddyNames.length;
		}

		/**
		 * @param row the element you want to get
		 * @return the Object at <tt>row</tt>
		*/
		public synchronized Object getElementAt( int row )
		{
			return buddyNames[row];
		}

		/**
		 * @param buddy the buddy to add
		*/
		public void addBuddy( String buddy )
		{
			buddies.add( buddy );
			fireChanged();
		}

		/**
		 * Removes a buddy from the list
		*/
		public void removeBuddy( String buddy )
		{
			int row = 0;
			boolean found = false;

			for( int i = 0; i < buddies.size(); i++ )
			{
				String item = (String)buddies.get( i );
				if( item.equals( buddy ) )
				{
					found = true;
					row = i;
				}
			}

			if( found )
			{
				buddies.remove( row );
				fireChanged();
			}
		}

		NickListModel thisPointer = this;

		/**
		 * Fires a change of the list
		*/
		private synchronized void fireChanged()
		{
			buddyNames = buddies.toArray();

			Arrays.sort( buddyNames );

			SwingUtilities.invokeLater( new Runnable()
			{
				public void run()
				{
					fireContentsChanged( thisPointer, 0, buddyNames.length );
					nickList.repaint();
					nickList.validate();
				}
			} );
		}
	}

	/**
	 * Listens for mouse events in the JList
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class DoubleClickListener extends MouseAdapter
	{
		public void mousePressed( MouseEvent e ) { checkPop( e ); }
		public void mouseReleased( MouseEvent e ) { checkPop( e ); }

		private boolean clicked = false;
		private javax.swing.Timer clickTimer = new javax.swing.Timer( 350, new TimerHandler() );

		/**
		 * Called when a click is recieved
		 * We have to hack this with a timer, we cannot use the regular method for
		 * double click detection, so we use a timer.  If we do not, we will have
		 * text box focus problems.  The text input box should _always_ have
		 * focus, so this is a way to ensure that.
		*/
		public void mouseClicked( MouseEvent e )
		{
			checkPop( e );
			JList list = (JList)e.getSource();

			if( clicked )
			{
				BuddyStatus buddy = window.getBuddyStatus( (String)list.getSelectedValue() );
				initiatePMConversation( buddy );
				clickTimer.stop();
				clicked = false;
			}
			else {
				clicked = true;
				clickTimer.start();
			}

			window.getTextEntryArea().grabFocus();
		}

		/**
		 * Shows the popup menu
		*/
		public void checkPop( MouseEvent e )
		{
			BuddyStatus buddy = null;

			if( e.isPopupTrigger() )
			{
				try {

					JList list = (JList)e.getComponent();
					int index = list.locationToIndex( e.getPoint() );
					list.setSelectedIndex( index );

					String user = (String)list.getSelectedValue();
					buddy = window.getBuddyStatus( user );
					popMenu.showMenu( e.getComponent(), e.getX(), e.getY(), buddy );
				}
				catch( ClassCastException ex ) { /*is not a buddy, so don't display the menu*/ }
			}
		}

		/**
		 * The time is up, the double click is off
		*/
		class TimerHandler implements ActionListener
		{
			public void actionPerformed( ActionEvent e )
			{
				clicked = false;
			}
		}
	}

	/**
	 * Starts a conversation with a buddy in a groupchat
	 * @param buddy the buddy to start the conversation with
	*/
	public void initiatePMConversation( BuddyStatus buddy )
	{
		String nick = "";
		String room = "";
		String parts[] = new String[2];
		parts = buddy.getUser().split( "\\/" );
		nick = parts[1];
		room = parts[0];

		if( buddy.getConversation() == null )
		{
			ChatPanel conver = new ChatPanel( buddy );
			buddy.setConversation( conver );
			MessageDelegator.getInstance().showPanel( buddy.getConversation() );
			MessageDelegator.getInstance().frontFrame( buddy.getConversation() );
		}
		else {
			MessageDelegator.getInstance().showPanel( buddy.getConversation() );
			MessageDelegator.getInstance().frontFrame( buddy.getConversation() );
		}

		buddy.getConversation().stopTimer();
	}
}
