/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.jabber.smack;

import java.util.*;
import java.util.regex.*;
import javax.swing.*;
import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.RosterEntry;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Packet;

import com.valhalla.jbother.*;
import com.valhalla.jbother.groupchat.*;
import com.valhalla.jbother.jabber.*;

/**
 * Listens for a message packet, and sends it to the appropriate buddies
 * conversation window.  If the window does not exist a new window will be created
 *
 * @author Adam Olsen
 * @version 1.0
 */

public class MessagePacketListener implements PacketListener
{
	private Hashtable buddyStatuses;  //all the available buddy statuses

	/**
	 * Constructor for the packet listener
	 */
	public MessagePacketListener()
	{
		buddyStatuses = BuddyList.getInstance().getBuddyStatuses();
	}

	/**
	 * Processes the message packet
	*/
	public void processPacket( Packet message )
	{
		final Message packet = (Message)message;

		if( packet.getType() == Message.Type.NORMAL )
		{
			if( packet.getBody() == null ) return;

			ParsedBuddyInfo info = new ParsedBuddyInfo( packet.getFrom() );
			String userId = info.getUserId().toLowerCase();
			if( BuddyList.getInstance().getBlockedUsers().containsKey( userId ) ) return;

			SwingUtilities.invokeLater( new Runnable()
			{
				public void run()
				{
					MessagePanel window = new MessagePanel();
					window.recieveMessage( packet );
				}
			} );

			return;
		}

		if( packet.getType() != Message.Type.CHAT && packet.getType() != Message.Type.HEADLINE ) return;

		String from = packet.getFrom();

		//check to see if it's a private message
		if( BuddyList.getInstance().getTabFrame() != null &&
				BuddyList.getInstance().getTabFrame().isRoomOpen( from.replaceAll( "\\/.*", "" ) ) )
		{
			ChatRoomPanel window = BuddyList.getInstance().getTabFrame().getChatPanel( from.replaceAll( "\\/.*", "" ) );
			if( window != null ) initiatePMSession( window, packet );
			return;
		}

		if( from != null )
		{
			ParsedBuddyInfo info = new ParsedBuddyInfo( from );
			String userId = info.getUserId().toLowerCase();
			final String resource = info.getResource();
			String server = info.getServer();
			from = info.getBareAddress();

			final BuddyStatus buddyStatus = BuddyList.getInstance().getBuddyStatus( userId );
			boolean inList = true;

			if( BuddyList.getInstance().getBlockedUsers().containsKey( userId ) )
			{
				com.valhalla.Logger.debug( "Blocking user: " + userId );
				return;
			}

			final String messageSbj  = packet.getSubject();
			final String messageBody = packet.getBody();

			if( !BuddyList.getInstance().checkConnection() ) return;

			// we don't want null messages to be displayed.
			if( ( messageBody == null ) ||
				( !BuddyList.getInstance().getBuddyListTree().getShowAgentMessages() && ( userId == from ) ) ) return;

			RosterEntry entry = BuddyList.getInstance().getConnection().getRoster().getEntry( from );
			if( entry != null ) userId = entry.getName();

			if( buddyStatus.getName() != null ) userId = buddyStatus.getName();

			if( buddyStatus.getConversation() == null )
			{
				  if (packet.getType() == Message.Type.HEADLINE)
				  	buddyStatus.setConversation( new HeadlinesPanel(buddyStatus) );
				  else
				  	buddyStatus.setConversation( new ChatPanel(buddyStatus) );
			}

			if( inList && buddyStatus.size() <= 0 )
				buddyStatus.getConversation().setOfflineMessage();

			SwingUtilities.invokeLater( new Runnable()
			{
				public void run()
				{
					buddyStatus.getConversation().recieveMessage( messageSbj, messageBody, resource );
				}
			} );
		}
	}

	/**
	 * If it's a group chat packet listener, process the packet
	 * @param window the chat room window
	 * @param packet the message
	*/
	private void initiatePMSession( ChatRoomPanel window, Message packet )
	{
		BuddyStatus buddy = window.getBuddyStatus( packet.getFrom() );

		String nick = "";
		String room = "";
		String parts[] = new String[2];
		parts = buddy.getUser().split( "\\/" );
		nick = parts[1];
		room = parts[0];

		if( buddy.getConversation() == null )
		{
			ChatPanel conver = new ChatPanel( buddy );
			conver.setVisible( true );
			buddy.setConversation( conver );
		}

		String messageBody = packet.getBody();
		if( messageBody == null ) return;

		buddy.getConversation().recieveMessage( "", messageBody, null );
	}
}
