/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.jabber.smack;

import java.util.regex.*;

import javax.swing.JOptionPane;

import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.RosterEntry;
import org.jivesoftware.smack.packet.Packet;
import org.jivesoftware.smack.packet.Presence;

import com.valhalla.jbother.*;
import com.valhalla.gui.*;
import com.valhalla.jbother.jabber.*;
import javax.swing.SwingUtilities;

import java.util.*;
import java.text.*;

/**
 * This class listens to all presence packets, and delivers them according to their type, etc...
 *
 * @author Adam Olsen
 * @version 1.0
**/

public class PresencePacketListener implements PacketListener
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private VersionCollectorThread vThread = null;
	private Properties currentlyRequesting = new Properties();

	/**
	 * Called when a Presence packet is recieved
	*/
	public void processPacket( Packet packet )
	{
		if( !BuddyList.getInstance().checkConnection() ) return;

		Presence presence = (Presence)packet;
		String from = presence.getFrom();

		ParsedBuddyInfo info = new ParsedBuddyInfo( from );
		String userId = info.getUserId();
		String resource = info.getResource();
		String server = info.getServer();

		// if the user is blocked, ignore this packet
		if( BuddyList.getInstance().getBlockedUsers().containsKey( userId ) ) return;

		// if this is a groupchat packet, then we don't care about it in this class
		if( BuddyList.getInstance().getTabFrame() != null &&
			BuddyList.getInstance().getTabFrame().getChatPanel( userId ) != null ) return;

		// if they are trying to subscribe, as if we want to let them subscribe
		if( presence.getType() == Presence.Type.SUBSCRIBE )
		{
			requestSubscription( userId );
			return;
		}

		// beyond this point, we don't care about anything but online and offline packets
		if( presence.getType() != Presence.Type.AVAILABLE && presence.getType() != Presence.Type.UNAVAILABLE ) return;

		BuddyStatus buddy = BuddyList.getInstance().getBuddyStatus( userId );

		boolean isSelf = false;
		if( buddy.getUser().equals( BuddyList.getInstance().getConnection().getUser().replaceAll( "/.*", "" ) ) ) isSelf = true;

		//if( !isSelf || true )
			performPresenceTasks( buddy, presence.getType() );


		// if it's unavailable, check to see if they have any resources still online
		// if they do, set the packet to available, and minus one resource
		if( presence.getType() == Presence.Type.UNAVAILABLE )
		{
			buddy.removeResource( resource );
			if( buddy.size() > 0 )
			{
				presence.setType( Presence.Type.AVAILABLE );
			}
			else {
				if( !BuddyList.getInstance().getBuddyListTree().getShowOfflineBuddies() )
				{
					BuddyList.getInstance().getBuddyListTree().removeBuddy( buddy, buddy.getGroup() );
				}
			}
		}
		else
		{
			int priority = presence.getPriority();
			if( priority < 0 ) priority = 0;

			buddy.addResource( resource, priority, presence.getMode(), presence.getStatus() );
		}

		if( !BuddyList.getInstance().checkConnection() ) return;

		if( isSelf )
		{
			SwingUtilities.invokeLater( new Runnable()
			{
				public void run()
				{
					BuddyList.getInstance().getTopMenu().getStatusMenu().loadSelfStatuses();
				}
			} );
		}

		ConversationPanel conv = buddy.getConversation();
		if( (conv != null) && (conv instanceof ChatPanel) ) ((ChatPanel)conv).updateResources();
	}

	/**
	 * Performs presence tasks, like playing sounds when a buddy signs on or off
	 * or displaying that they have done so in their conversation window if it exists
	 * @param buddy the buddy to process
	 * @param type the presence type
	*/
	public void performPresenceTasks( final BuddyStatus buddy, final Presence.Type type )
	{
		SwingUtilities.invokeLater( new Runnable()
		{
			public void run()
			{
				ConversationPanel conv = buddy.getConversation();

				if( type == Presence.Type.AVAILABLE && !buddy.getHasSignedOn() )
				{
					buddy.setHasSignedOn( true );
					if( (conv != null) && (conv instanceof ChatPanel) ) ((ChatPanel)conv).signedOn();
					com.valhalla.jbother.sound.SoundPlayer.play( "signedOnSound" );
				}

				if( type == Presence.Type.UNAVAILABLE && buddy.size() <= 1 && buddy.getHasSignedOn() )
				{
					if( (conv != null) && (conv instanceof ChatPanel) ) ((ChatPanel)conv).signedOff();
					buddy.setHasSignedOn( false );
					com.valhalla.jbother.sound.SoundPlayer.play( "signedOffSound" );
				}
			}
		} );
	}

	/**
	 * If someone sends a subscription request packet, the user will be asked if they
	 * want to accept the request
	 * @param userId the requesting user
	*/
	public void requestSubscription( final String userId )
	{
		SwingUtilities.invokeLater( new Runnable()
		{
			public void run()
			{
				if( currentlyRequesting.getProperty( userId.toLowerCase() ) != null ) return;
				currentlyRequesting.setProperty( userId.toLowerCase(), "true" );

				NMOptionDialog d = new NMOptionDialog( BuddyList.getInstance(),
					resources.getString( "subscriptionRequest" ),
					MessageFormat.format( resources.getString( "requestedPresence" ), new Object[] { userId } ), NMOptionDialog.QUESTION );

				d.addButton( "Yes", 1 );
				d.addButton( "No", 2 );
				d.show();

				d.addOptionListener( new NMOptionListener()
				{
					public void buttonClicked( int num )
					{
						currentlyRequesting.remove( userId );
						if( num == 1 )
						{
							Presence packet = new Presence( Presence.Type.SUBSCRIBED );
							packet.setTo( userId );

							BuddyList.getInstance().getConnection().sendPacket( packet );
							BuddyStatus buddy = BuddyList.getInstance().getBuddyStatus( userId );
							buddy.setRemoved( false );
						}

						boolean add = true;
						// find out if they are already in the roster
						if( BuddyList.getInstance().getConnection().getRoster().contains( userId.toLowerCase() ) ) add = false;
						if( userId.indexOf( "@" ) < 0 ) add = false;

						if( add )
						{
							NMOptionDialog di = new NMOptionDialog( BuddyList.getInstance(),
								resources.getString( "addButton" ),
								MessageFormat.format( resources.getString( "doAddBuddy" ), new Object[] { userId } ), NMOptionDialog.QUESTION );

							di.addButton( "Yes", 1 );
							di.addButton( "No", 2 );
							di.show();

							di.addOptionListener( new NMOptionListener()
							{
								public void buttonClicked( int num )
								{
									if( num == 1 )
									{
										AddBuddyDialog dialog = new AddBuddyDialog();
										dialog.setBuddyId( userId );
										dialog.setVisible( true );
									}
								}
							} );
						}
					}

				} );
			}
		} );
	}
}
