/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.preferences;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Constructor;
import java.util.*;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;

import com.valhalla.gui.*;
import com.valhalla.pluginmanager.*;
import com.valhalla.jbother.*;
import com.valhalla.settings.*;

/**
 * This is the preferences dialog.  It is basically a container for the different
 * PreferencesPanels, which are swapped in and out as the user selects items
 * from the preferences tree.
 * @author Adam Olsen
 * @version 1.0
 **/
public class PreferencesDialog extends JDialog
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private PreferencesTree tree;
	private JPanel container = new JPanel();
	private JPanel rightSide = new JPanel();
	private JPanel buttonPanel = new JPanel();
	private JPanel prefsPanel = new JPanel( new GridLayout( 0, 1 ) );

	private JButton		cancelButton = new JButton( resources.getString( "cancelButton" ) ),
						okButton = new JButton( resources.getString( "okButton" ) ),
						applyButton = new JButton( resources.getString( "applyButton" ) );

	private boolean ok = true;
	private JPanel currentPrefsPanel[];
	private int currentIndex = 0;
	private PreferencesDialog thisPointer = this;

	/**
	 * Creates the PreferencesDialog
	*/
	public PreferencesDialog()
	{
		super( BuddyList.getInstance(), "Preferences", false );
		setTitle( resources.getString( "preferences" ) );
		initComponents();

		container.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
		pack();
		setLocationRelativeTo( null );
		DialogTracker.addDialog( this, false, true );
	}

	/**
	 * Sets up the visual components
	*/
	private void initComponents()
	{
		container.setLayout( new BoxLayout( container, BoxLayout.X_AXIS ) );

		tree = new PreferencesTree( this );
		currentPrefsPanel = new JPanel[tree.getRowCount()];
		container.add( tree );

		rightSide.setLayout( new BoxLayout( rightSide, BoxLayout.Y_AXIS ) );
		container.add( rightSide );

		// set the general preferences panel as the default selected panel
		currentPrefsPanel[0] = new GeneralPreferencesPanel( this );
		prefsPanel.add( currentPrefsPanel[0] );
		rightSide.add( prefsPanel );

		buttonPanel.setLayout( new BoxLayout( buttonPanel, BoxLayout.X_AXIS ) );
		buttonPanel.add( Box.createHorizontalGlue() );
		buttonPanel.add( cancelButton );
		buttonPanel.add( applyButton );
		buttonPanel.add( okButton );

		rightSide.add( Box.createRigidArea( new Dimension( 0, 5 ) ) );
		rightSide.add( buttonPanel );
		rightSide.setPreferredSize( new Dimension( 520, 400 ) );

		//set up the handlers
		PrefsActionHandler handler = new PrefsActionHandler();
		cancelButton.addActionListener( handler );
		okButton.addActionListener( handler );
		applyButton.addActionListener( handler );

		setContentPane( container );
	}

	/**
	 * Listens for events on the different buttons
	*/
	class PrefsActionHandler implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			if( e.getSource() == cancelButton )
			{
				DialogTracker.removeDialog( thisPointer );
			}

			if( e.getSource() == okButton ) writeSettings();
			if( e.getSource() == applyButton ) applyHandler();
		}
	}

	/**
	 * Applies the settings
	*/
	private void applyHandler()
	{
		PreferencesPanel panel = (PreferencesPanel)currentPrefsPanel[currentIndex];
		applySettings( panel.getSettings() );
		Standard.noticeMessage( this, resources.getString( "applySettings" ),
			resources.getString( "settingsHaveBeenApplied" ) );
	}

	/**
	 * Applies the specified Settings
	 * @param settings the settings to apply
	*/
	private void applySettings( TempSettings settings )
	{
		Iterator iterator = settings.keySet().iterator();

		while( iterator.hasNext() )
		{
			String key = (String)iterator.next();

			if( settings.getProperty( key ).equals( "!!REMOVED!!" ) )
				Settings.getInstance().setBoolean( key, false );
			else Settings.getInstance().setProperty( key, settings.getProperty( key ) );

		}

		BuddyList.getInstance().updateIcons();

		// check the away timer
		if( Settings.getInstance().getBoolean( "autoAway" ) )
			BuddyList.getInstance().getAwayTimer().start();
		else
			BuddyList.getInstance().getAwayTimer().stop();

	}

	/**
	 * Switches the currently displayed preferences panel to the specified one
	 * @param string the panel to switch to
	*/
	protected void switchPanel( String string )
	{
		prefsPanel.remove( currentPrefsPanel[currentIndex] );
		currentIndex = tree.getSelectedIndex();

		// if there has already been a panel assigned to the selected index,
		// use it.
		if( currentPrefsPanel[currentIndex] != null )
		{
			prefsPanel.add( currentPrefsPanel[currentIndex] );
			prefsPanel.repaint();
			validate();
			return;
		}

		// otherwise, dynamically get the name of the selected panel and create an instance
		// matching the name of the class
		try {
			Class def = null;

			if( Arguments.getInstance().getProperty( "webstart" ) == null )
			{
				def = PluginLoader.getInstance().loadClass(
					"com.valhalla.jbother.preferences." + string + "PreferencesPanel" );
			}
			else {
				def = getClass().getClassLoader().loadClass(
					"com.valhalla.jbother.preferences." + string + "PreferencesPanel" );
			}

			Class[] thisArgsClass = new Class[] { PreferencesDialog.class };
			Object[] thisArgs = new Object[] { this };

			Constructor con = def.getConstructor( thisArgsClass );
			currentPrefsPanel[currentIndex] = (JPanel)con.newInstance( thisArgs );
			prefsPanel.add( currentPrefsPanel[currentIndex] );
			validate();
		}
		catch( ClassNotFoundException ex )
		{
			com.valhalla.Logger.debug( "Cannot find that preferences class." );
		}
		catch( NoSuchMethodException ex )
		{
			com.valhalla.Logger.debug( "Could not find constructor." );
		}
		catch( Exception e )
		{
			com.valhalla.Logger.logException( e );
			com.valhalla.Logger.debug( "Could not create preferences class." );
		}
	}

	/**
	 * Writes the settings to the settings file
	*/
	private void writeSettings()
	{
		for( int i = 0; i < currentPrefsPanel.length; i++ )
		{
			JPanel panel = currentPrefsPanel[i];
			if( panel != null )
			{
				PreferencesPanel prefsPanel = (PreferencesPanel)panel;
				applySettings( prefsPanel.getSettings() );
			}
		}

		DialogTracker.removeDialog( thisPointer );
	}


}
