/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.settings;

import java.io.*;
import java.util.*;

/**
 * A singleton Properties class to access and save settings
 *
 * @author Adam Olsen
 * @version 1.0
**/
public class Settings extends Properties
{
	private	static Settings instance;
	private File settingsDir;
	private File settingsFile;
	private java.util.Timer writeTimer = new java.util.Timer();

	/**
	 * Reads the settings file and sets up the singleton
	 *
	 * @param dir the settings directory
	 * @param settingsFile the settings file
	**/
	public static void loadSettings( String dir, String settingsFile )
	{
		instance = new Settings( dir, settingsFile );
	}

	/**
	 * Gets the Settings instance
	 * @return the Settings singleton
	*/
	public static Settings getInstance() { return instance; }

	/**
	 * Gets the settings directory
	 * @return the settings directory
	**/
	public File getSettingsDir() { return settingsDir; }

	/**
	 * Gets the settings file
	 *
	 * @return the settings file
	**/
	public File getSettingsFile() { return settingsFile; }

	/**
	 * Private constructor
	 *
	 * @param dir the settings directory
	 * @param settingsFile the settings file
	**/
	private Settings( String dir, String settingsFile )
	{
		instance = this;
		settingsDir = new File( dir );
		this.settingsFile = new File( dir, settingsFile );

		// make sure the settings directory and file is there
		if( !settingsDir.isDirectory() || !this.settingsFile.isFile() ) createDefaultSettings();

		try {
			// load the file into the properties
			InputStream stream = new FileInputStream( this.settingsFile );
			load( stream );
			stream.close();

			writeTimer.schedule( new TimerTask()
			{
				public void run() { writeSettings(); }
			}, 10000, 10000 );
		}
		catch( Exception e )
		{
			com.valhalla.Logger.debug( "Could not load settings file" );
			com.valhalla.Logger.debug( e.getMessage() );
		}
	}

	/**
	 * Gets a boolean value
	 * @param key the key to get a boolean for
	 * @return a boolean based on the key
	**/
	public boolean getBoolean( String key )
	{
		return ( getProperty( key  ) != null );
	}

	/**
	 * Sets a boolean
	 * @param key the key to set
	 * @param value the value to set the key to
	**/
	public void setBoolean( String key, boolean value )
	{
		if( value ) setProperty( key, "true" );
		else remove( key );
	}

	/**
	 * Sets the settings
	 * @param settings the settings to set it to
	**/
	public static void setSettings( Properties settings )
	{
		if( instance != null ) instance = (Settings)settings;
	}

	/**
	 * Writes the settings to the settings file
	**/
	public static void writeSettings()
	{
		if( instance == null )
		{
			com.valhalla.Logger.debug( "Fatal Error: Settings has not been initiated, and an attempt to access a value was made. Potential NullPointerException." );
			com.valhalla.Logger.closeLog();
			System.exit( 1 );
		}

		try {
			// create the output stream
			FileOutputStream stream = new FileOutputStream( instance.settingsFile );
			StringBuffer buf = new StringBuffer();

			// store the properties file
			instance.store( stream, "JBother Settings File" );
		}
		catch( IOException e )
		{
			com.valhalla.Logger.debug( "There was an error saving your settings." );
			com.valhalla.Logger.debug( e.toString() );
		}
	}

	/**
	 * If a settings file is not found, a default settings file is copied in to place.
	**/
	private void createDefaultSettings()
	{
		if( !settingsDir.isDirectory() && !settingsDir.mkdirs() )
		{
			//creating the settings directory failed....
			com.valhalla.Logger.debug( "Fatal Error: Could not create the settings directory (" + settingsDir.getName() + ")." );
			com.valhalla.Logger.closeLog();
		}

		try {

			// copy the default file in to place
			InputStream stream = instance.getClass().getClassLoader().getResourceAsStream( "defaultsettings.properties" );
			if( stream == null ) throw new IOException();

			BufferedReader in = new BufferedReader( new InputStreamReader( stream ) );

			PrintWriter out = new PrintWriter( new FileWriter( settingsFile ) );
			String line = null;

			while( ( line = in.readLine() ) != null )
			{
				out.println( line );
			}

			in.close();
			out.close();

		}
		catch( IOException ex ) {
			//creating an initial settings file failed...
			com.valhalla.Logger.debug( "Fatal Error: Could not create the settings file (" +
				settingsFile.getPath() + ")\n" + ex.getMessage() );
			com.valhalla.Logger.closeLog();
			System.exit( 1 );
		}
	}
}